@@ -1,5 +1,5 @@
 (* circe2/float.ml --  *)
-(* Copyright (C) 2001-2018 by Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
+(* Copyright (C) 2001-2019 by Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
    Circe2 is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by 
    the Free Software Foundation; either version 2, or (at your option)
@@ -39,13 +39,13 @@ module Double =
     let little_endian = true
 
     let to_string x =
-      let s = sprintf "%.17E" x in
-      for i = 0 to String.length s - 1 do
-        let c = s.[i] in
+      let s = Bytes.of_string (sprintf "%.17E" x) in
+      for i = 0 to Bytes.length s - 1 do
+        let c = Bytes.get s i in
         if c = 'e' || c = 'E' then
-          s.[i] <- 'D'
+          Bytes.set s i 'D'
       done;
-      s
+      Bytes.to_string s
 
     (* Identity floatingpoint numbers that are indistinguishable from
        integers for more concise printing. *)
