@@ -255,11 +255,11 @@ let assert_command
          in
          let () =
            (* Dump sinput into the process stdin *)
-           let buff = " " in
+           let buff = Bytes.make 1 ' ' in
              Stream.iter 
                (fun c ->
                   let _i : int =
-                    buff.[0] <- c;
+                    Bytes.set buff 0 c;
                     Unix.write out_write buff 0 1
                   in
                     ())
@@ -291,11 +291,11 @@ let assert_command
                let chn = 
                  open_in fn_out
                in
-               let buff = String.make 4096 'X' in
+               let buff = Bytes.make 4096 'X' in
                let len = ref (-1) in
                  while !len <> 0 do 
-                   len := input chn buff 0 (String.length buff);
-                   printf "%s" (String.sub buff 0 !len);
+                   len := input chn buff 0 (Bytes.length buff);
+                   printf "%s" (Bytes.sub_string buff 0 !len);
                  done;
                  printf "@?";
                  close_in chn
