@@ -1,6 +1,6 @@
 (* vertex.ml --
 
-   Copyright (C) 1999-2018 by
+   Copyright (C) 1999-2019 by
 
        Wolfgang Kilian <kilian@physik.uni-siegen.de>
        Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
@@ -47,6 +47,9 @@ module Expr =
 	  UFOx_lexer.token
 	  (UFOx_lexer.init_position "" (Lexing.from_string text))
       with
+      | UFO_tools.Lexical_Error (msg, start_pos, end_pos) ->
+	 invalid_arg (Printf.sprintf "lexical error (%s) at: `%s'"
+			msg  (error_in_string text start_pos end_pos))
       | UFOx_syntax.Syntax_Error (msg, start_pos, end_pos) ->
 	 invalid_arg (Printf.sprintf "syntax error (%s) at: `%s'"
 			msg  (error_in_string text start_pos end_pos))
@@ -62,29 +65,41 @@ module Expr =
 
     open UFOx_syntax
 
-    let rec substitute name value = function
-      | Integer _ | Float _ as e -> e
+    let rec map f = function
+      | Integer _ | Float _ | Quoted _ as e -> e
       | Variable s as e ->
-         if s = name then
-           value
-         else
-           e
-      | Sum (e1, e2) ->
-         Sum (substitute name value e1, substitute name value e2)
-      | Difference (e1, e2) ->
-         Difference (substitute name value e1, substitute name value e2)
-      | Product (e1, e2) ->
-         Product (substitute name value e1, substitute name value e2)
-      | Quotient (e1, e2) ->
-         Quotient (substitute name value e1, substitute name value e2)
-      | Power (e1, e2) ->
-         Power (substitute name value e1, substitute name value e2)
-      | Application (s, el) ->
-         Application (s, List.map (substitute name value) el)
+         begin match f s with
+         | Some value -> value
+         | None -> e
+         end
+      | Sum (e1, e2) -> Sum (map f e1, map f e2)
+      | Difference (e1, e2) -> Difference (map f e1, map f e2)
+      | Product (e1, e2) -> Product (map f e1, map f e2)
+      | Quotient (e1, e2) -> Quotient (map f e1, map f e2)
+      | Power (e1, e2) -> Power (map f e1, map f e2)
+      | Application (s, el) -> Application (s, List.map (map f) el)
+
+    let substitute name value expr =
+      map (fun s -> if s = name then Some value else None) expr
+
+    module SMap = Map.Make (struct type t = string let compare = compare end)
+
+    let rename1 name_map name =
+      try Some (Variable (SMap.find name name_map)) with Not_found -> None
+
+    let rename alist_names value =
+      let name_map =
+        List.fold_left
+          (fun acc (name, name') -> SMap.add name name' acc)
+          SMap.empty alist_names in
+      map (rename1 name_map) value
 
     let half name =
       Quotient (Variable name, Integer 2)
 
+    let variables = UFOx_syntax.variables
+    let functions = UFOx_syntax.functions
+
   end
 
 let positive integers =
@@ -93,16 +108,21 @@ let positive integers =
 let not_positive integers =
   List.filter (fun (i, _) -> i <= 0) integers
 
-let int_list_to_string is =
-  "[" ^ String.concat ", " (List.map string_of_int is) ^ "]"
-	
-module Q = Algebra.Small_Rational
-
 module type Index =
   sig
+
+    val position : int -> int
+    val factor : int -> int
+    val unpack : int -> int * int
+    val pack : int -> int -> int
+    val map_position : (int -> int) -> int -> int
+    val to_string : int -> string
+    val list_to_string : int list -> string
+
     val free : (int * 'r) list -> (int * 'r) list
     val summation : (int * 'r) list -> (int * 'r) list
     val classes_to_string : ('r -> string) -> (int * 'r) list -> string
+
   end
 
 module Index : Index =
@@ -111,6 +131,49 @@ module Index : Index =
     let free i = positive i
     let summation i = not_positive i
 
+    let position i =
+      if i > 0 then
+        i mod 1000
+      else
+        i
+
+    let factor i =
+      if i > 0 then
+        i / 1000
+      else
+        invalid_arg "UFOx.Index.factor: argument not positive"
+
+    let unpack i =
+      if i > 0 then
+        (position i, factor i)
+      else
+        (i, 0)
+
+    let pack i j =
+      if j > 0 then
+        if i > 0 then
+          1000 * j + i
+        else
+          invalid_arg "UFOx.Index.pack: position not positive"
+      else if j = 0 then
+        i
+      else
+        invalid_arg "UFOx.Index.pack: factor negative"
+
+    let map_position f i =
+      let pos, fac = unpack i in
+      pack (f pos) fac
+
+    let to_string i =
+      let pos, fac = unpack i in
+      if fac = 0 then
+        Printf.sprintf "%d" pos
+      else
+        Printf.sprintf "%d.%d" pos fac
+
+    let list_to_string is =
+      "[" ^ String.concat ", " (List.map to_string is) ^ "]"
+	
     let classes_to_string rep_to_string index_classes =
       let reps =
 	ThoList.uniq (List.sort compare (List.map snd index_classes)) in
@@ -119,7 +182,7 @@ module Index : Index =
 	(List.map
 	   (fun r ->
 	     (rep_to_string r) ^ "=" ^
-	       (int_list_to_string
+	       (list_to_string
 		  (List.map
 		     fst
 		     (List.filter (fun (_, r') -> r = r') index_classes))))
@@ -130,12 +193,14 @@ module Index : Index =
 module type Atom =
   sig
     type t
+    val map_indices : (int -> int) -> t -> t
     val of_expr : string -> UFOx_syntax.expr list -> t
     val to_string : t -> string
     type r
     val classify_indices : t list -> (int * r) list
     val rep_to_string : r -> string
-    val rep_of_int : int -> r
+    val rep_to_string_whizard : r -> string
+    val rep_of_int : bool -> int -> r
     val rep_conjugate : r -> r
     val rep_trivial : r -> bool
     type r_omega
@@ -145,7 +210,9 @@ module type Atom =
 module type Tensor =
   sig
     type atom
-    type t = (atom list * Q.t) list
+    type t = (atom list * Algebra.QC.t) list
+    val map_atoms : (atom -> atom) -> t -> t
+    val map_indices : (int -> int) -> t -> t
     val of_expr : UFOx_syntax.expr -> t
     val of_string : string -> t
     val of_strings : string list -> t
@@ -153,7 +220,8 @@ module type Tensor =
     type r
     val classify_indices : t -> (int * r) list 
     val rep_to_string : r -> string
-    val rep_of_int : int -> r
+    val rep_to_string_whizard : r -> string
+    val rep_of_int : bool -> int -> r
     val rep_conjugate : r -> r
     val rep_trivial : r -> bool
     type r_omega
@@ -165,26 +233,39 @@ module Tensor (A : Atom) : Tensor
   struct
 
     module S = UFOx_syntax
+    (* TODO: we have to switch to [Algebra.QC] to support complex
+       coefficients, as used in custom propagators. *)
+    module Q = Algebra.Q
+    module QC = Algebra.QC
 
     type atom = A.t
-    type t = (atom list * Q.t) list
+    type t = (atom list * QC.t) list
+
+    let map_atoms f t =
+      List.map (fun (atoms, q) -> (List.map f atoms, q)) t
+
+    let map_indices f t =
+      map_atoms (A.map_indices f) t
 
     let multiply (t1, c1) (t2, c2) =
-      (List.sort compare (t1 @ t2), Q.mul c1 c2)
+      (List.sort compare (t1 @ t2), QC.mul c1 c2)
 
     let compress terms =
-      List.map (fun (t, cs) -> (t, Q.sum cs)) (ThoList.factorize terms)
+      List.map (fun (t, cs) -> (t, QC.sum cs)) (ThoList.factorize terms)
 
     let rec of_expr e =
       compress (of_expr' e)
 
     and of_expr' = function
-      | S.Integer i -> [([], Q.make i 1)]
+      | S.Integer i -> [([], QC.make (Q.make i 1) Q.null)]
       | S.Float _ -> invalid_arg "UFOx.Tensor.of_expr: unexpected float"
+      | S.Quoted name ->
+	 invalid_arg ("UFOx.Tensor.of_expr: unexpected quoted variable '" ^
+			 name ^ "'")
       | S.Variable name ->
 	 invalid_arg ("UFOx.Tensor.of_expr: unexpected variable '" ^
 			 name ^ "'")
-      | S.Application (name, args) -> [([A.of_expr name args], Q.unit)]
+      | S.Application (name, args) -> [([A.of_expr name args], QC.unit)]
       | S.Sum (e1, e2) ->
 	 of_expr e1 @ of_expr e2
       | S.Difference (e1, e2) ->
@@ -193,7 +274,7 @@ module Tensor (A : Atom) : Tensor
       | S.Quotient (n, d) ->
 	 begin match of_expr d with
 	 | [([], q)] ->
-	    List.map (fun (t, c) -> (t, Q.div c q)) (of_expr n)
+	    List.map (fun (t, c) -> (t, QC.div c q)) (of_expr n)
 	 | [] ->
 	    failwith "UFOx.Tensor.of_expr: zero denominator"
 	 | _ ->
@@ -202,17 +283,28 @@ module Tensor (A : Atom) : Tensor
       | S.Power (e, p) ->
 	 begin match of_expr e, of_expr p with
 	 | [([], q)], [([], p)] ->
-	    if Q.is_integer p then
-	      [([], Q.pow q (Q.to_integer p))]
-	    else
-	      failwith "UFOx.Tensor.of_expr: rational power"
+	    if QC.is_real p then
+              let re_p = QC.real p in
+	      if Q.is_integer re_p then
+	        [([], QC.pow q (Q.to_integer re_p))]
+	      else
+	        failwith "UFOx.Tensor.of_expr: rational power"
+            else
+	      failwith "UFOx.Tensor.of_expr: complex power"
 	 | [([], q)], _ ->
 	    failwith "UFOx.Tensor.of_expr: non-numeric power"
+	 | t, [([], p)] ->
+            let qc = QC.sub p (QC.make (Q.make 2 1) Q.null) in
+            if QC.is_null qc then
+              Product.list2 multiply t t
+            else
+	      failwith "UFOx.Tensor.of_expr: only 2 as power of tensor allowed"
 	 | _ -> failwith "UFOx.Tensor.of_expr: power of tensor"
 	 end
 
     type r = A.r
     let rep_to_string = A.rep_to_string
+    let rep_to_string_whizard = A.rep_to_string_whizard
     let rep_of_int = A.rep_of_int
     let rep_conjugate = A.rep_conjugate
     let rep_trivial = A.rep_trivial
@@ -222,21 +314,24 @@ module Tensor (A : Atom) : Tensor
 	(List.sort compare
 	   (List.map (fun (t, c) -> filter (A.classify_indices t)) tensors))
 
+    (* NB: the number of summation indices is not guarateed to be
+       the same!  Therefore it was foolish to try to check for
+       uniqueness \ldots *)
     let classify_indices tensors =
-      let free_indices = classify_indices' Index.free tensors
-      and summation_indices = classify_indices' Index.summation tensors in
-      match free_indices, summation_indices with
-      | [], _ -> failwith "UFOx.Tensor.classify_indices: can't happen!"
-      | [f], [s] -> f
-      | [_], _ ->
-	 invalid_arg
-	   "UFOx.Tensor.classify_indices: superfluous summation indices!"
-      | _, _ ->
+      match classify_indices' Index.free tensors with
+      | [] ->
+         (* There's always at least an empty list! *)
+         failwith "UFOx.Tensor.classify_indices: can't happen!"
+      | [f] -> f
+      | _ ->
 	 invalid_arg "UFOx.Tensor.classify_indices: incompatible free indices!"
 
+    let check_indices t =
+      ignore (classify_indices t)
+
     let of_expr e =
       let t = of_expr e in
-      let free = classify_indices t in
+      check_indices t;
       t
 
     let of_string s =
@@ -246,33 +341,15 @@ module Tensor (A : Atom) : Tensor
       of_expr (Expr.of_strings s)
 
     let term_to_string (tensors, c) =
-      if Q.is_null c then
+      if QC.is_null c then
 	""
       else
-	(if Q.is_negative c then " - " else " + ") ^
-	  (let c = Q.abs c in
-	   if Q.is_unit c && tensors = [] then
-	     ""
-	   else
-	     Q.to_string c) ^
-	  (match tensors with
-	  | [] -> ""
-	  | tensors ->
-	     (if Q.is_unit (Q.abs c) then "" else "*") ^
-	       String.concat "*" (List.map A.to_string tensors))
-
-    let term_to_string (tensors, c) =
-      if Q.is_null c then
-	""
-      else
-	(if Q.is_negative c then " - " else " + ") ^
-	  (let c = Q.abs c in
-	   match tensors with
-	   | [] -> Q.to_string c
-	   | tensors ->
-	      String.concat "*"
-		((if Q.is_unit c then [] else [Q.to_string c]) @
-		    List.map A.to_string tensors))
+	match tensors with
+	| [] -> QC.to_string c
+	| tensors ->
+	   String.concat
+             "*" ((if QC.is_unit c then [] else [QC.to_string c]) @
+		    List.map A.to_string tensors)
 
     let to_string terms =
       String.concat "" (List.map term_to_string terms)
@@ -284,32 +361,56 @@ module Tensor (A : Atom) : Tensor
 
 module type Lorentz_Atom =
   sig
-    type t = private
+
+    type dirac = private
       | C of int * int
-      | Epsilon of int * int * int * int
       | Gamma of int * int * int
       | Gamma5 of int * int
       | Identity of int * int
-      | Metric of int * int
-      | P of int * int
       | ProjP of int * int
       | ProjM of int * int
       | Sigma of int * int * int * int
+
+    type vector = (* private *)
+      | Epsilon of int * int * int * int
+      | Metric of int * int
+      | P of int * int
+
+    type t = private
+      | Dirac of dirac
+      | Vector of vector
+
+    val map_indices_vector : (int -> int) -> vector -> vector
+
   end
 
 module Lorentz_Atom =
   struct
-    type t =
+
+    type dirac =
       | C of int * int
-      | Epsilon of int * int * int * int
       | Gamma of int * int * int
       | Gamma5 of int * int
       | Identity of int * int
-      | Metric of int * int
-      | P of int * int
       | ProjP of int * int
       | ProjM of int * int
       | Sigma of int * int * int * int
+
+    (* TODO: the propagators use additional atoms! *)
+    type vector =
+      | Epsilon of int * int * int * int
+      | Metric of int * int
+      | P of int * int
+
+    type t =
+      | Dirac of dirac
+      | Vector of vector
+
+    let map_indices_vector f = function
+      | Epsilon (mu, nu, ka, la) -> Epsilon (f mu, f nu, f ka, f la)
+      | Metric (mu, nu) -> Metric (f mu, f nu)
+      | P (mu, n) -> P (f mu, f n)
+
   end
 
 module Lorentz_Atom' : Atom
@@ -320,21 +421,28 @@ module Lorentz_Atom' : Atom
 
     open Lorentz_Atom
     
-    let to_string = function
+    let map_indices_dirac f = function
+      | C (i, j) -> C (f i, f j)
+      | Gamma (mu, i, j) -> Gamma (f mu, f i, f j)
+      | Gamma5 (i, j) -> Gamma5 (f i, f j)
+      | Identity (i, j) -> Identity (f i, f j)
+      | ProjP (i, j) -> ProjP (f i, f j)
+      | ProjM (i, j) -> ProjM (f i, f j)
+      | Sigma (mu, nu, i, j) -> Sigma (f mu, f nu, f i, f j)
+
+    let map_indices f = function
+      | Dirac d -> Dirac (map_indices_dirac f d)
+      | Vector v -> Vector (map_indices_vector f v)
+
+    let dirac_to_string = function
       | C (i, j) ->
 	 Printf.sprintf "C(%d,%d)" i j
-      | Epsilon (mu, nu, ka, la) ->
-	 Printf.sprintf "Epsilon(%d,%d,%d,%d)" mu nu ka la
       | Gamma (mu, i, j) ->
 	 Printf.sprintf "Gamma(%d,%d,%d)" mu i j
       | Gamma5 (i, j) ->
 	 Printf.sprintf "Gamma5(%d,%d)" i j
       | Identity (i, j) ->
 	 Printf.sprintf "Identity(%d,%d)" i j
-      | Metric (mu, nu) ->
-	 Printf.sprintf "Metric(%d,%d)" mu nu
-      | P (mu, n) ->
-	 Printf.sprintf "P(%d,%d)" mu n
       | ProjP (i, j) ->
 	 Printf.sprintf "ProjP(%d,%d)" i j
       | ProjM (i, j) ->
@@ -342,95 +450,167 @@ module Lorentz_Atom' : Atom
       | Sigma (mu, nu, i, j) ->
 	 Printf.sprintf "Sigma(%d,%d,%d,%d)" mu nu i j
 
+    let vector_to_string = function
+      | Epsilon (mu, nu, ka, la) ->
+	 Printf.sprintf "Epsilon(%d,%d,%d,%d)" mu nu ka la
+      | Metric (mu, nu) ->
+	 Printf.sprintf "Metric(%d,%d)" mu nu
+      | P (mu, n) ->
+	 Printf.sprintf "P(%d,%d)" mu n
+
+    let to_string = function
+      | Dirac d -> dirac_to_string d
+      | Vector v -> vector_to_string v
+
     module S = UFOx_syntax
 
     let of_expr name args =
       match name, args with
-      | "C", [S.Integer i; S.Integer j] -> C (i, j)
+      | "C", [S.Integer i; S.Integer j] -> Dirac (C (i, j))
       | "C", _ ->
 	 invalid_arg "UFOx.Lorentz.of_expr: invalid arguments to C()"
       | "Epsilon", [S.Integer mu; S.Integer nu; S.Integer ka; S.Integer la] ->
-	 Epsilon (mu, nu, ka, la)
+	 Vector (Epsilon (mu, nu, ka, la))
       | "Epsilon", _ ->
 	 invalid_arg "UFOx.Lorentz.of_expr: invalid arguments to Epsilon()"
       | "Gamma", [S.Integer mu; S.Integer i; S.Integer j] ->
-	 Gamma (mu, i, j)
+	 Dirac (Gamma (mu, i, j))
       | "Gamma", _ ->
 	 invalid_arg "UFOx.Lorentz.of_expr: invalid arguments to Gamma()"
-      | "Gamma5", [S.Integer i; S.Integer j] -> Gamma5 (i, j)
+      | "Gamma5", [S.Integer i; S.Integer j] -> Dirac (Gamma5 (i, j))
       | "Gamma5", _ ->
 	 invalid_arg "UFOx.Lorentz.of_expr: invalid arguments to Gamma5()"
-      | "Identity", [S.Integer i; S.Integer j] -> Identity (i, j)
+      | "Identity", [S.Integer i; S.Integer j] -> Dirac (Identity (i, j))
       | "Identity", _ ->
 	 invalid_arg "UFOx.Lorentz.of_expr: invalid arguments to Identity()"
-      | "Metric", [S.Integer mu; S.Integer nu] -> Metric (mu, nu)
+      | "Metric", [S.Integer mu; S.Integer nu] -> Vector (Metric (mu, nu))
       | "Metric", _ ->
 	 invalid_arg "UFOx.Lorentz.of_expr: invalid arguments to Metric()"
-      | "P", [S.Integer mu; S.Integer n] -> P (mu, n)
+      | "P", [S.Integer mu; S.Integer n] -> Vector (P (mu, n))
       | "P", _ ->
 	 invalid_arg "UFOx.Lorentz.of_expr: invalid arguments to P()"
-      | "ProjP", [S.Integer i; S.Integer j] -> ProjP (i, j)
+      | "ProjP", [S.Integer i; S.Integer j] -> Dirac (ProjP (i, j))
       | "ProjP", _ ->
 	 invalid_arg "UFOx.Lorentz.of_expr: invalid arguments to ProjP()"
-      | "ProjM", [S.Integer i; S.Integer j] -> ProjM (i, j)
+      | "ProjM", [S.Integer i; S.Integer j] -> Dirac (ProjM (i, j))
       | "ProjM", _ ->
 	 invalid_arg "UFOx.Lorentz.of_expr: invalid arguments to ProjM()"
       | "Sigma", [S.Integer mu; S.Integer nu; S.Integer i; S.Integer j] ->
-	 Sigma (mu, nu, i, j)
+         if mu <> nu then
+	   Dirac (Sigma (mu, nu, i, j))
+         else
+	   invalid_arg "UFOx.Lorentz.of_expr: implausible arguments to Sigma()"
       | "Sigma", _ ->
 	 invalid_arg "UFOx.Lorentz.of_expr: invalid arguments to Sigma()"
       | name, _ ->
 	 invalid_arg ("UFOx.Lorentz.of_expr: invalid tensor '" ^ name ^ "'")
 
-    type r = S | V | Sp | CSp | Ghost
+    type r = S | V | T | Sp | CSp | Maj | Ghost
 
     let rep_trivial = function
       | S | Ghost -> true
-      | V | Sp | CSp-> false
+      | V | T | Sp | CSp | Maj -> false
 
     let rep_to_string = function
       | S -> "0"
       | V -> "1"
+      | T -> "2"
       | Sp -> "1/2"
       | CSp-> "1/2bar"
+      | Maj -> "1/2M"
       | Ghost -> "Ghost"
 
-    let rep_of_int = function
+    let rep_to_string_whizard = function
+      | S -> "0"
+      | V -> "1"
+      | T -> "2"
+      | Sp | CSp | Maj -> "1/2"
+      | Ghost -> "Ghost"
+
+    let rep_of_int neutral = function
       | -1 -> Ghost
       | 1 -> S
-      | 2 -> Sp
+      | 2 -> if neutral then Maj else Sp
+      | -2 -> if neutral then Maj else CSp (* used by [UFO.Particle.force_conjspinor] *)
       | 3 -> V
-      | _ -> invalid_arg "UFOx.Lorentz: impossible representation!"
+      | 4 -> failwith "UFOx.Lorentz: spin 3/2 not supported yet!"
+      | 5 -> T
+      | s when s > 0 ->
+         failwith "UFOx.Lorentz: spin > 2 not supported!"
+      | _ ->
+         invalid_arg "UFOx.Lorentz: invalid non-positive spin value"
 	 
     let rep_conjugate = function
       | S -> S
       | V -> V
+      | T -> T
       | Sp -> CSp (* ??? *)
       | CSp -> Sp (* ??? *)
+      | Maj -> Maj
       | Ghost -> Ghost
 
-    let classify_indices1 = function
+    let classify_vector_indices1 = function
+      | Epsilon (mu, nu, ka, la) -> [(mu, V); (nu, V); (ka, V); (la, V)]
+      | Metric (mu, nu) -> [(mu, V); (nu, V)]
+      | P (mu, n) ->  [(mu, V)]
+
+    let classify_dirac_indices1 = function
       | C (i, j) -> [(i, CSp); (j, Sp)] (* ??? *)
       | Gamma5 (i, j) | Identity (i, j)
       | ProjP (i, j) | ProjM (i, j) -> [(i, CSp); (j, Sp)]
-      | Epsilon (mu, nu, ka, la) -> [(mu, V); (nu, V); (ka, V); (la, V)]
       | Gamma (mu, i, j) -> [(mu, V); (i, CSp); (j, Sp)]
-      | Metric (mu, nu) -> [(mu, V); (nu, V)]
-      | P (mu, n) ->  [(mu, V)]
       | Sigma (mu, nu, i, j) -> [(mu, V); (nu, V); (i, CSp); (j, Sp)]
 
-    let classify_indices tensors =
-      List.sort compare
-	(List.fold_right
-	   (fun v acc -> classify_indices1 v @ acc)
-	   tensors [])
+    let classify_indices1 = function
+      | Dirac d -> classify_dirac_indices1 d
+      | Vector v -> classify_vector_indices1 v
+
+    module IMap = Map.Make (struct type t = int let compare = compare end)
+
+    exception Incompatible_factors
+
+    let product rep1 rep2 =
+      match rep1, rep2 with
+      | V, V -> T
+      | _, _ -> raise Incompatible_factors
+
+    let combine_or_add_index (i, rep) map =
+      let pos, fac = Index.unpack i in
+      try
+        let fac', rep' = IMap.find pos map in
+        if pos < 0 then
+          IMap.add pos (fac, rep) map
+        else if fac <> fac' then
+          IMap.add pos (0, product rep rep') map
+        else
+          invalid_arg (Printf.sprintf "UFO: duplicate subindex %d" pos)
+      with
+      | Not_found -> IMap.add pos (fac, rep) map
+      | Incompatible_factors ->
+         invalid_arg (Printf.sprintf "UFO: incompatible factors at %d" pos)
+
+    let combine_or_add_indices atom map =
+      List.fold_right combine_or_add_index (classify_indices1 atom) map
+
+    let project_factors (pos, (fac, rep)) =
+      if fac = 0 then
+        (pos, rep)
+      else
+        invalid_arg (Printf.sprintf "UFO: leftover subindex %d.%d" pos fac)
+
+    let classify_indices atoms =
+      List.map
+        project_factors
+        (IMap.bindings (List.fold_right combine_or_add_indices atoms IMap.empty))
 
     type r_omega = Coupling.lorentz
     let omega = function
       | S -> Coupling.Scalar
       | V -> Coupling.Vector
+      | T -> Coupling.Tensor_2
       | Sp -> Coupling.Spinor
-      | CSp-> Coupling.ConjSpinor
+      | CSp -> Coupling.ConjSpinor
+      | Maj -> Coupling.Majorana
       | Ghost -> Coupling.Scalar
 
   end
@@ -439,8 +619,9 @@ module Lorentz = Tensor(Lorentz_Atom')
 
 module type Color_Atom =
   sig
-    type t = private
+    type t = (* private *)
       | Identity of int * int
+      | Identity8 of int * int
       | T of int * int * int
       | F of int * int * int
       | D of int * int * int
@@ -455,6 +636,7 @@ module Color_Atom =
   struct
     type t =
       | Identity of int * int
+      | Identity8 of int * int
       | T of int * int * int
       | F of int * int * int
       | D of int * int * int
@@ -475,6 +657,18 @@ module Color_Atom' : Atom
 
     open Color_Atom
 
+    let map_indices f = function
+      | Identity (i, j) -> Identity (f i, f j)
+      | Identity8 (a, b) -> Identity8 (f a, f b)
+      | T (a, i, j) -> T (f a, f i, f j)
+      | F (a, i, j) -> F (f a, f i, f j)
+      | D (a, i, j) -> D (f a, f i, f j)
+      | Epsilon (i, j, k) -> Epsilon (f i, f j, f k)
+      | EpsilonBar (i, j, k) -> EpsilonBar (f i, f j, f k)
+      | T6 (a, i', j') -> T6 (f a, f i', f j')
+      | K6 (i', j, k) -> K6 (f i', f j, f k)
+      | K6Bar (i', j, k) -> K6Bar (f i', f j, f k)
+
     let of_expr name args =
       match name, args with
       | "Identity", [S.Integer i; S.Integer j] -> Identity (i, j)
@@ -511,6 +705,7 @@ module Color_Atom' : Atom
 	
     let to_string = function
       | Identity (i, j) -> Printf.sprintf "Identity(%d,%d)" i j
+      | Identity8 (a, b) -> Printf.sprintf "Identity8(%d,%d)" a b
       | T (a, i, j) -> Printf.sprintf "T(%d,%d,%d)" a i j
       | F (a, b, c) -> Printf.sprintf "f(%d,%d,%d)" a b c
       | D (a, b, c) -> Printf.sprintf "d(%d,%d,%d)" a b c
@@ -533,7 +728,14 @@ module Color_Atom' : Atom
       | C -> "3bar"
       | A-> "8"
 
-    let rep_of_int = function
+    let rep_to_string_whizard = function
+      | S -> "1"
+      | Sbar -> "-1"
+      | F -> "3"
+      | C -> "-3"
+      | A-> "8"
+
+    let rep_of_int neutral = function
       | 1 -> S
       | -1 -> Sbar (* UFO appears to use this for colorless antiparticles!. *)
       | 3 -> F
@@ -551,6 +753,7 @@ module Color_Atom' : Atom
 
     let classify_indices1 = function
       | Identity (i, j) -> [(i, C); (j, F)]
+      | Identity8 (a, b) -> [(a, A); (b, A)]
       | T (a, i, j) -> [(i, F); (j, C); (a, A)]
       | Color_Atom.F (a, b, c) | D (a, b, c) -> [(a, A); (b, A); (c, A)] 
       | Epsilon (i, j, k) -> [(i, F); (j, F); (k, F)]
@@ -570,6 +773,7 @@ module Color_Atom' : Atom
 
     type r_omega = Color.t
 
+    (* FIXME: $N_C=3$ should not be hardcoded! *)
     let omega = function
       | S | Sbar -> Color.Singlet
       | F -> Color.SUN (3)
@@ -584,23 +788,65 @@ module Value =
   struct
 
     module S = UFOx_syntax
+    module Q = Algebra.Q
 
     type builtin =
       | Sqrt
-      | Cos
-      | Sin
+      | Exp | Log | Log10
+      | Sin | Asin
+      | Cos | Acos
+      | Tan | Atan
+      | Sinh | Asinh
+      | Cosh | Acosh
+      | Tanh | Atanh
+      | Sec | Asec
+      | Csc | Acsc
       | Conj
 
     let builtin_to_string = function
       | Sqrt -> "sqrt"
-      | Cos -> "cos"
+      | Exp -> "exp"
+      | Log -> "log"
+      | Log10 -> "log10"
       | Sin -> "sin"
+      | Cos -> "cos"
+      | Tan -> "tan"
+      | Asin -> "asin"
+      | Acos -> "acos"
+      | Atan -> "atan"
+      | Sinh -> "sinh"
+      | Cosh -> "cosh"
+      | Tanh -> "tanh"
+      | Asinh -> "asinh"
+      | Acosh -> "acosh"
+      | Atanh -> "atanh"
+      | Sec -> "sec"
+      | Csc -> "csc"
+      | Asec -> "asec"
+      | Acsc -> "acsc"
       | Conj -> "conjg"
 
     let builtin_of_string = function
       | "cmath.sqrt" -> Sqrt
-      | "cmath.cos" -> Cos
+      | "cmath.exp" -> Exp
+      | "cmath.log" -> Log
+      | "cmath.log10" -> Log10
       | "cmath.sin" -> Sin
+      | "cmath.cos" -> Cos
+      | "cmath.tan" -> Tan
+      | "cmath.asin" -> Asin
+      | "cmath.acos" -> Acos
+      | "cmath.atan" -> Atan
+      | "cmath.sinh" -> Sinh
+      | "cmath.cosh" -> Cosh
+      | "cmath.tanh" -> Tanh
+      | "cmath.asinh" -> Asinh
+      | "cmath.acosh" -> Acosh
+      | "cmath.atanh" -> Atanh
+      | "sec" -> Sec
+      | "csc" -> Csc
+      | "asec" -> Asec
+      | "acsc" -> Acsc
       | "complexconjugate" -> Conj
       | name -> failwith ("UFOx.Value: unsupported function: " ^ name)
 
@@ -665,17 +911,24 @@ module Value =
       | e -> "(" ^ to_string e ^ ")"
 
     let rec to_coupling atom = function
-      | Integer i -> Coupling.Const i
+      | Integer i -> Coupling.Integer i
       | Rational q ->
          let n, d = Q.to_ratio q in
-         Coupling.Quot (Coupling.Const n, Coupling.Const d)
-      | Real x -> Coupling.Atom (atom (string_of_float x))
+         Coupling.Quot (Coupling.Integer n, Coupling.Integer d)
+      | Real x -> Coupling.Float x
       | Product es -> Coupling.Prod (List.map (to_coupling atom) es)
       | Variable s -> Coupling.Atom (atom s)
+      | Complex (r, 0.0) -> Coupling.Float r
+      | Complex (0.0,  1.0) -> Coupling.I
+      | Complex (0.0, -1.0) -> Coupling.Prod [Coupling.I; Coupling.Integer (-1)]
+      | Complex (0.0, i) -> Coupling.Prod [Coupling.I; Coupling.Float i]
+      | Complex (r, 1.0) ->
+         Coupling.Sum [Coupling.Float r; Coupling.I]
+      | Complex (r, -1.0) ->
+         Coupling.Diff (Coupling.Float r, Coupling.I)
       | Complex (r, i) ->
-         Coupling.Sum [Coupling.Atom (atom (string_of_float r));
-                       Coupling.Prod [Coupling.I;
-                                      Coupling.Atom (atom (string_of_float i))]]
+         Coupling.Sum [Coupling.Float r;
+                       Coupling.Prod [Coupling.I; Coupling.Float i]]
       | Sum es -> Coupling.Sum (List.map (to_coupling atom) es)
       | Difference (e1, e2) ->
          Coupling.Diff (to_coupling atom e1, to_coupling atom e2)
@@ -683,14 +936,41 @@ module Value =
          Coupling.Quot (to_coupling atom e1, to_coupling atom e2)
       | Power (e1, Integer e2) ->
          Coupling.Pow (to_coupling atom e1, e2)
-      | Application (Sin, [e]) -> Coupling.Sin (to_coupling atom e)
-      | Application (Cos, [e]) -> Coupling.Cos (to_coupling atom e)
-      | Application (Sqrt, [e]) -> Coupling.Sqrt (to_coupling atom e)
-      | Application (Conj, [e]) -> Coupling.Conj (to_coupling atom e)
-      | Power (e1, _) ->
-         invalid_arg "UFOx.Value.to_coupling: non-integer power"
-      | Application (_, _) ->
-         failwith "UFOx.Value.to_coupling: more than one argument list"
+      | Power (e1, e2) ->
+         Coupling.PowX (to_coupling atom e1, to_coupling atom e2)
+      | Application (f, [e]) -> apply1 (to_coupling atom e) f
+      | Application (f, []) ->
+         failwith
+           ("UFOx.Value.to_coupling:  " ^ builtin_to_string f ^
+              ": empty argument list")
+      | Application (f, _::_::_) ->
+         failwith
+           ("UFOx.Value.to_coupling: " ^ builtin_to_string f ^
+              ": more than one argument in list")
+
+    and apply1 e = function
+      | Sqrt -> Coupling.Sqrt e
+      | Exp -> Coupling.Exp e
+      | Log -> Coupling.Log e
+      | Log10 -> Coupling.Log10 e
+      | Sin -> Coupling.Sin e
+      | Cos -> Coupling.Cos e
+      | Tan -> Coupling.Tan e
+      | Asin -> Coupling.Asin e
+      | Acos -> Coupling.Acos e
+      | Atan -> Coupling.Atan e
+      | Sinh -> Coupling.Sinh e
+      | Cosh -> Coupling.Cosh e
+      | Tanh -> Coupling.Tanh e
+      | Sec -> Coupling.Quot (Coupling.Integer 1, Coupling.Cos e)
+      | Csc -> Coupling.Quot (Coupling.Integer 1, Coupling.Sin e)
+      | Asec -> Coupling.Acos (Coupling.Quot (Coupling.Integer 1, e))
+      | Acsc -> Coupling.Asin (Coupling.Quot (Coupling.Integer 1, e))
+      | Conj -> Coupling.Conj e
+      | (Asinh | Acosh | Atanh as f) ->
+         failwith
+           ("UFOx.Value.to_coupling: function `"
+            ^ builtin_to_string f ^ "' not supported yet!")
 
     let compress terms = terms
 
@@ -701,6 +981,9 @@ module Value =
       | S.Integer i -> Integer i
       | S.Float x -> Real x
       | S.Variable "cmath.pi" -> Variable "pi"
+      | S.Quoted name ->
+	 invalid_arg ("UFOx.Value.of_expr: unexpected quoted variable '" ^
+			 name ^ "'")
       | S.Variable name -> Variable name
       | S.Sum (e1, e2) ->
 	 begin match of_expr e1, of_expr e2 with
