@@ -1,6 +1,6 @@
 (* combinatorics.ml --
 
-   Copyright (C) 1999-2018 by
+   Copyright (C) 1999-2019 by
 
        Wolfgang Kilian <kilian@physik.uni-siegen.de>
        Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
@@ -22,6 +22,10 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  *)
 
+(* Avoid refering to [Pervasives.compare], because [Pervasives] will
+   become [Stdlib.Pervasives] in O'Caml 4.07 and [Stdlib] in O'Caml 4.08. *)
+let pcompare = compare
+
 type 'a seq = 'a list
 
 (* \thocwmodulesection{Simple Combinatorial Functions} *)
@@ -361,6 +365,21 @@ let permute_even l =
 let permute_odd l =
   filter_sign (-1) (permute_signed l)
 
+(* \begin{dubious}
+     We have a slight inconsistency here:
+     [permute [] = [[]]], while
+     [permute_cyclic [] = []].
+     I don't know if it is worth fixing.
+   \end{dubious} *)
+
+let permute_cyclic l =
+  let rec permute_cyclic' acc l1 = function
+    | [] -> List.rev acc
+    | x :: rest as l2 ->
+       permute_cyclic' ((l2 @ List.rev l1) :: acc) (x :: l1) rest
+  in
+  permute_cyclic' [] [] l
+
 (* \thocwmodulesubsection{Tensor Products of Permutations} *)
 
 let permute_tensor ll =
@@ -395,14 +414,14 @@ let insert_inorder_signed order x (eps, l) =
   in
   insert 1 [] l
 
-let sort_signed ?(cmp=Pervasives.compare) l =
+let sort_signed ?(cmp=pcompare) l =
   List.fold_right (insert_inorder_signed cmp) l (1, [])
 
-let sign ?(cmp=Pervasives.compare) l =
+let sign ?(cmp=pcompare) l =
   let eps, _ = sort_signed ~cmp l in
   eps
 
-let sign2 ?(cmp=Pervasives.compare) l =
+let sign2 ?(cmp=pcompare) l =
   let a = Array.of_list l in
   let eps = ref 1 in
   for j = 0 to Array.length a - 1 do
@@ -418,6 +437,75 @@ module Test =
 
     open OUnit
 
+    let to_string =
+      ThoList.to_string (ThoList.to_string string_of_int)
+
+    let assert_equal_perms =
+      assert_equal ~printer:to_string
+
+    let count_permutations n =
+      let factorial_n = factorial n
+      and range = ThoList.range 1 n in
+      let sorted = List.sort compare (permute range) in
+      (* Verify the count \ldots *)
+      assert_equal factorial_n (List.length sorted);
+      (* \ldots{} check that they're all different \ldots *)
+      assert_equal factorial_n (List.length (ThoList.uniq sorted));
+      (* \ldots{} make sure that they a all permutations. *)
+      assert_equal_perms
+        [range] (ThoList.uniq (List.map (List.sort compare) sorted))
+
+    let suite_permute =
+      "permute" >:::
+	[ "permute []" >::
+	    (fun () ->
+              assert_equal_perms [[]] (permute []));
+          "permute [1]" >::
+	    (fun () ->
+              assert_equal_perms [[1]] (permute [1]));
+          "permute [1;2;3]" >::
+	    (fun () ->
+              assert_equal_perms
+                [ [2; 3; 1]; [2; 1; 3]; [3; 2; 1];
+                  [1; 3; 2]; [1; 2; 3]; [3; 1; 2] ]
+                (permute [1; 2; 3]));
+          "permute [1;2;3;4]" >::
+	    (fun () ->
+              assert_equal_perms
+                [ [3; 4; 1; 2]; [3; 1; 2; 4]; [3; 1; 4; 2];
+                  [4; 3; 1; 2]; [1; 4; 2; 3]; [1; 2; 3; 4];
+                  [1; 2; 4; 3]; [4; 1; 2; 3]; [1; 4; 3; 2];
+                  [1; 3; 2; 4]; [1; 3; 4; 2]; [4; 1; 3; 2];
+                  [3; 4; 2; 1]; [3; 2; 1; 4]; [3; 2; 4; 1];
+                  [4; 3; 2; 1]; [2; 4; 1; 3]; [2; 1; 3; 4];
+                  [2; 1; 4; 3]; [4; 2; 1; 3]; [2; 4; 3; 1];
+                  [2; 3; 1; 4]; [2; 3; 4; 1]; [4; 2; 3; 1] ]
+                (permute [1; 2; 3; 4]));
+          "count permute 5" >::
+            (fun () -> count_permutations 5);
+          "count permute 6" >::
+            (fun () -> count_permutations 6);
+          "count permute 7" >::
+            (fun () -> count_permutations 7);
+          "count permute 8" >::
+            (fun () -> count_permutations 8);
+          "cyclic []" >::
+	    (fun () ->
+              assert_equal_perms [] (permute_cyclic []));
+          "cyclic [1]" >::
+	    (fun () ->
+              assert_equal_perms [[1]] (permute_cyclic [1]));
+          "cyclic [1;2;3]" >::
+	    (fun () ->
+	      assert_equal_perms
+                [[1;2;3]; [2;3;1]; [3;1;2]]
+                (permute_cyclic [1;2;3]));
+          "cyclic [1;2;3;4]" >::
+	    (fun () ->
+	      assert_equal_perms
+                [[1;2;3;4]; [2;3;4;1]; [3;4;1;2]; [4;1;2;3]]
+                (permute_cyclic [1;2;3;4]))]
+
     let sort_signed_not_unique =
       "not unique" >::
 	(fun () ->
@@ -468,7 +556,8 @@ module Test =
 
     let suite =
       "Combinatorics" >:::
-	[suite_sort_signed]
+	[suite_permute;
+         suite_sort_signed]
 
   end
 
