@@ -1,6 +1,6 @@
 (* omega.ml --
 
-   Copyright (C) 1999-2018 by
+   Copyright (C) 1999-2019 by
 
        Wolfgang Kilian <kilian@physik.uni-siegen.de>
        Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
@@ -65,6 +65,9 @@ module Make (Fusion_Maker : Fusion.Maker) (Target_Maker : Target.Maker) (M : Mod
     module W = Whizard.Make(Fusion_Maker)(P)(P_Whizard)(M)
     module C = Cascade.Make(M)(P)
 
+    module VSet =
+      Set.Make (struct type t = F.constant Coupling.t let compare = compare end)
+
 (* For the phase space, we need asymmetric DAGs.
 
    HACK: since we will not use this to compute amplitudes, there's
@@ -154,10 +157,6 @@ i*)
 
     let dag_sans_color = function
       | amplitude :: _ as amplitudes ->
-	let prune_color wf =
-	  (F.flavor_sans_color wf, F.momentum_list wf) in
-	let prune_color_and_couplings (wf, c) =
-	  (prune_color wf, None) in
         let prune a = a in
         List.map prune amplitudes
       | [] -> []
@@ -512,6 +511,13 @@ i*)
             CF.amplitudes (include_goldstones !checks) !unphysical_polarization
 	      CF.no_exclusions selectors processes
           with
+          | Fusion.Majorana ->
+             begin
+               Printf.eprintf
+                 "O'Mega: found Majorana fermions: use a supporting binary!\n";
+               flush stderr;
+               CF.empty;
+             end
           | exc ->
               begin 
                 Printf.eprintf
@@ -533,6 +539,29 @@ i*)
               Printf.eprintf ", %d diagrams" (F.count_diagrams amplitude);
               Printf.eprintf "\n")
             (CF.processes amplitudes);
+          let couplings =
+            List.fold_left
+              (fun acc p ->
+                let fusions = ThoList.flatmap F.rhs (F.fusions p)
+                and brakets = ThoList.flatmap F.ket (F.brakets p) in
+                let couplings =
+                  VSet.of_list (List.map F.coupling (fusions @ brakets)) in
+                VSet.union acc couplings)
+              VSet.empty (CF.processes amplitudes) in
+          Printf.eprintf "SUMMARY: %d vertices\n" (VSet.cardinal couplings);
+          let ufo_couplings =
+            VSet.fold
+              (fun v acc ->
+                match v with
+                | Coupling.Vn (Coupling.UFO (_, v, _, _, _), _, _) ->
+                   Sets.String.add v acc
+                | _ -> acc)
+              couplings Sets.String.empty in
+          if not (Sets.String.is_empty ufo_couplings) then
+            Printf.eprintf
+              "SUMMARY: %d UFO vertices: %s\n"
+              (Sets.String.cardinal ufo_couplings)
+              (String.concat ", " (Sets.String.elements ufo_couplings))
         end;
 
         if !poles then begin
