@@ -1,6 +1,6 @@
 (* omega_UFO.ml --
 
-   Copyright (C) 1999-2018 by
+   Copyright (C) 1999-2019 by
 
        Wolfgang Kilian <kilian@physik.uni-siegen.de>
        Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
@@ -22,7 +22,16 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  *)
 
-module O = Omega.Make(Fusion.Mixed23)(Targets.Fortran)(UFO.Model)
+module Bound (M : Model.T) : Tuple.Bound =
+  struct
+    (* \begin{dubious}
+         Above [max_degree = 6], the performance drops \emph{dramatically}!
+       \end{dubious} *)
+    let max_arity () =
+      pred (M.max_degree ())
+  end
+
+module O = Omega.Make(Fusion.Nary(Bound(UFO.Model)))(Targets.Fortran)(UFO.Model)
 let _ = O.main ()
 
 (*i
