@@ -1,6 +1,6 @@
 (* thoList.ml --
 
-   Copyright (C) 1999-2018 by
+   Copyright (C) 1999-2019 by
 
        Wolfgang Kilian <kilian@physik.uni-siegen.de>
        Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
@@ -22,6 +22,10 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  *)
 
+(* Avoid refering to [Pervasives.compare], because [Pervasives] will
+   become [Stdlib.Pervasives] in O'Caml 4.07 and [Stdlib] in O'Caml 4.08. *)
+let pcompare = compare
+
 let rec hdn n l =
   if n <= 0 then
     []
@@ -52,6 +56,11 @@ let splitn n l =
   else
     splitn' n [] l
 
+let split_last l =
+  match List.rev l with
+  | [] -> invalid_arg "ThoList.split_last []"
+  | ln :: l12_rev -> (List.rev l12_rev, ln)
+
 (* This is [splitn'] all over again, but without the exception. *)
 let rec chopn'' n l1_rev l2 =
   if n <= 0 then
@@ -76,6 +85,33 @@ let chopn n l =
   else
     chopn' n [] l
 
+(* Find a member [a] in the list [l] and return the
+   cyclically permuted list with [a] as head. *)
+let cycle_until a l =
+  let rec cycle_until' acc = function
+    | [] -> raise Not_found
+    | a' :: l' as al' ->
+       if a' = a then
+         al' @ List.rev acc
+       else
+         cycle_until' (a' :: acc) l' in
+  cycle_until' [] l
+
+let rec cycle' i acc l =
+  if i <= 0 then
+    l @ List.rev acc
+  else
+    match l with
+    | [] -> invalid_arg "ThoList.cycle"
+    | a' :: l' ->
+       cycle' (pred i) (a' :: acc) l'
+
+let cycle n l =
+  if n < 0 then
+    invalid_arg "ThoList.cycle"
+  else
+    cycle' n [] l
+
 let of_subarray n1 n2 a =
   let rec of_subarray' n1 n2 =
     if n1 > n2 then
@@ -103,6 +139,19 @@ let enumerate ?(stride=1) n l =
       (n, []) l in
   List.rev l_rev
 
+(* Take the elements of [list] that satisfy [predicate] and
+   form a list of pairs of an offset into the original list
+   and the element with the offsets
+   starting from [offset].  NB: the order of the returned alist
+   is not specified! *)
+let alist_of_list ?(predicate=(fun _ -> true)) ?(offset=0) list =
+  let _, alist =
+    List.fold_left
+      (fun (n, acc) x ->
+	(succ n, if predicate x then (n, x) :: acc else acc))
+      (offset, []) list in
+  alist
+
 (* This is \emph{not} tail recursive! *)
 let rec flatmap f = function
   | [] -> []
@@ -142,9 +191,13 @@ let transpose lists =
   try
     transpose' lists
   with
-  | Failure "tl" -> invalid_arg "ThoList.transpose: not rectangular"
+  | Failure s ->
+     if s = "tl" then
+       invalid_arg "ThoList.transpose: not rectangular"
+     else
+       failwith ("ThoList.transpose: unexpected Failure(" ^ s ^ ")")
 
-let compare ?(cmp=Pervasives.compare) l1 l2 =
+let compare ?(cmp=pcompare) l1 l2 =
   let rec compare' l1' l2' =
     match l1', l2' with
     | [], [] -> 0
@@ -179,6 +232,25 @@ let rec homogeneous = function
       else
         homogeneous rest
           
+let rec pairs' acc = function
+  | [] -> acc
+  | [x] -> invalid_arg "pairs: odd number of elements"
+  | x :: y :: indices ->
+     if x <> y then
+       invalid_arg "pairs: not in pairs"
+     else
+       begin match acc with
+       | [] -> pairs' [x] indices
+       | x' :: _ ->
+          if x = x' then
+            invalid_arg "pairs: more than twice"
+          else
+            pairs' (x :: acc) indices
+       end
+
+let pairs l =
+  pairs' [] (List.sort pcompare l)
+
 (* If we needed it, we could use a polymorphic version of [Set] to
    speed things up from~$O(n^2)$ to~$O(n\ln n)$.  But not before it
    matters somewhere \ldots *)
@@ -247,28 +319,24 @@ let rec rev_multiply n rl l =
 
 let multiply n l = rev_multiply n (List.rev l) l
 
-module ISet = Set.Make (struct type t = int let compare = Pervasives.compare end)
-
 exception Overlapping_indices
 exception Out_of_bounds
 
-let iset_of_list list =
-  List.fold_right ISet.add list ISet.empty
-
 let iset_list_union list =
-  List.fold_right ISet.union list ISet.empty
+  List.fold_right Sets.Int.union list Sets.Int.empty
 
 let complement_index_sets n index_set_lists =
-  let index_sets = List.map iset_of_list index_set_lists in
+  let index_sets = List.map Sets.Int.of_list index_set_lists in
   let index_set = iset_list_union index_sets in
   let size_index_sets =
-    List.fold_left (fun acc s -> ISet.cardinal s + acc) 0 index_sets in
-  if size_index_sets <> ISet.cardinal index_set then
+    List.fold_left (fun acc s -> Sets.Int.cardinal s + acc) 0 index_sets in
+  if size_index_sets <> Sets.Int.cardinal index_set then
     raise Overlapping_indices
-  else if ISet.exists (fun i -> i < 0 || i >= n) index_set then
+  else if Sets.Int.exists (fun i -> i < 0 || i >= n) index_set then
     raise Overlapping_indices
   else
-    match ISet.elements (ISet.diff (iset_of_list (range 0 (pred n))) index_set) with
+    match Sets.Int.elements
+            (Sets.Int.diff (Sets.Int.of_list (range 0 (pred n))) index_set) with
     | [] -> index_set_lists
     | complement -> complement :: index_set_lists
 
@@ -284,7 +352,7 @@ let partitioned_sort cmp index_sets list =
     () (complement_index_sets (List.length list) index_sets);
   Array.to_list array
 
-let ariadne_sort ?(cmp=Pervasives.compare) list =
+let ariadne_sort ?(cmp=pcompare) list =
   let sorted =
     List.sort (fun (n1, a1) (n2, a2) -> cmp a1 a2) (enumerate 0 list) in
   (List.map snd sorted, List.map fst sorted)
@@ -292,10 +360,10 @@ let ariadne_sort ?(cmp=Pervasives.compare) list =
 let ariadne_unsort (sorted, indices) =
   List.map snd
     (List.sort
-       (fun (n1, a1) (n2, a2) -> Pervasives.compare n1 n2)
+       (fun (n1, a1) (n2, a2) -> pcompare n1 n2)
        (List.map2 (fun n a -> (n, a)) indices sorted))
 
-let lexicographic ?(cmp=Pervasives.compare) l1 l2 =
+let lexicographic ?(cmp=pcompare) l1 l2 =
   let rec lexicographic' = function
     | [], [] -> 0
     | [], _ -> -1
@@ -319,11 +387,125 @@ let common l1 l2 =
 	acc)
     [] l1
 
-let complement l1 l2 =
-  if List.for_all (fun x -> List.mem x l1) l2 then
-    List.filter (fun x -> not (List.mem x l2)) l1
-  else
-    invalid_arg "ThoList.complement"
+let complement l1 = function
+  | [] -> l1
+  | l2 ->
+     if List.for_all (fun x -> List.mem x l1) l2 then
+       List.filter (fun x -> not (List.mem x l2)) l1
+     else
+       invalid_arg "ThoList.complement"
+
+
+let to_string a2s alist =
+  "[" ^ String.concat "; " (List.map a2s alist) ^ "]"
+
+let random_int_list imax n =
+  let imax_plus = succ imax in
+  Array.to_list (Array.init n (fun _ -> Random.int imax_plus))
+
+module Test =
+  struct
+
+    open OUnit
+
+    let suite_split =
+      "split*" >:::
+	[ "split_last []" >::
+	    (fun () ->
+	      assert_raises
+                (Invalid_argument "ThoList.split_last []")
+                (fun () -> split_last []));
+          "split_last [1]" >::
+	    (fun () ->
+	      assert_equal
+                ([], 1)
+                (split_last [1]));
+          "split_last [2;3;1;4]" >::
+	    (fun () ->
+	      assert_equal
+                ([2;3;1], 4)
+                (split_last [2;3;1;4])) ]
+
+    let test_list = random_int_list 1000 100
+
+    let assert_equal_int_list =
+      assert_equal ~printer:(to_string string_of_int)
+
+    let suite_cycle =
+      "cycle_until" >:::
+	[ "cycle (-1) [1;2;3]" >::
+	    (fun () ->
+	      assert_raises
+                (Invalid_argument "ThoList.cycle")
+                (fun () -> cycle 4 [1;2;3]));
+          "cycle 4 [1;2;3]" >::
+	    (fun () ->
+	      assert_raises
+                (Invalid_argument "ThoList.cycle")
+                (fun () -> cycle 4 [1;2;3]));
+          "cycle 42 [...]" >::
+	    (fun () ->
+              let n = 42 in
+	      assert_equal_int_list
+                (tln n test_list @ hdn n test_list)
+                (cycle n test_list));
+          "cycle_until 1 []" >::
+	    (fun () ->
+	      assert_raises
+                (Not_found)
+                (fun () -> cycle_until 1 []));
+          "cycle_until 1 [2;3;4]" >::
+	    (fun () ->
+	      assert_raises
+                (Not_found)
+                (fun () -> cycle_until 1 [2;3;4]));
+          "cycle_until 1 [1;2;3;4]" >::
+	    (fun () ->
+	      assert_equal
+                [1;2;3;4]
+                (cycle_until 1 [1;2;3;4]));
+          "cycle_until 3 [1;2;3;4]" >::
+	    (fun () ->
+	      assert_equal
+                [3;4;1;2]
+                (cycle_until 3 [3;4;1;2]));
+          "cycle_until 4 [1;2;3;4]" >::
+	    (fun () ->
+	      assert_equal
+                [4;1;2;3]
+                (cycle_until 4 [4;1;2;3])) ]
+
+    let suite_alist_of_list =
+      "alist_of_list" >:::
+	[ "simple" >::
+	    (fun () ->
+	      assert_equal
+                [(46, 4); (44, 2); (42, 0)]
+                (alist_of_list
+                   ~predicate:(fun n -> n mod 2 = 0) ~offset:42 [0;1;2;3;4;5])) ]
+
+    let suite_complement =
+      "complement" >:::
+	[ "simple" >::
+	    (fun () ->
+	      assert_equal [2;4] (complement [1;2;3;4] [1; 3]));
+          "empty" >::
+	    (fun () ->
+	      assert_equal [1;2;3;4] (complement [1;2;3;4] []));
+          "failure" >::
+	    (fun () ->
+              assert_raises
+                (Invalid_argument ("ThoList.complement"))
+	        (fun () -> complement (complement [1;2;3;4] [5]))) ]
+
+    let suite =
+      "ThoList" >:::
+	[suite_split;
+         suite_cycle;
+         suite_alist_of_list;
+         suite_complement]
+
+  end
 
 (*i
  *  Local Variables:
