@@ -1,6 +1,6 @@
 (* vertex.ml --
 
-   Copyright (C) 1999-2018 by
+   Copyright (C) 1999-2019 by
 
        Wolfgang Kilian <kilian@physik.uni-siegen.de>
        Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
@@ -22,6 +22,10 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  *)
 
+(* Avoid refering to [Pervasives.compare], because [Pervasives] will
+   become [Stdlib.Pervasives] in O'Caml 4.07 and [Stdlib] in O'Caml 4.08. *)
+let pcompare = compare
+
 module type Test =
   sig
     val example : unit -> unit
@@ -1245,7 +1249,7 @@ module Lorentz : Lorentz =
           Map.Make
             (struct
               type t = int * int
-              let compare = Pervasives.compare
+              let compare = pcompare
             end)
             
         let init2 triples =
@@ -1265,7 +1269,7 @@ module Lorentz : Lorentz =
           Map.Make
             (struct
               type t = int * (int * int)
-              let compare = Pervasives.compare
+              let compare = pcompare
             end)
             
         let init3 quadruples =
@@ -1289,7 +1293,7 @@ module Lorentz : Lorentz =
           Map.Make
             (struct
               type t = int * int * (int * int)
-              let compare = Pervasives.compare
+              let compare = pcompare
             end)
             
         let init4 quadruples =
@@ -1508,7 +1512,8 @@ module Test (M : Model.T) : Test =
           List.iter
             (Printf.printf "  type(momentum), intent(in) :: %s\n")
             momenta;
-          let [rhs1; rhs2] = rhs in
+          let rhs1 = List.hd rhs
+          and rhs2 = List.hd (List.tl rhs) in
           begin match M.lorentz (M.flavor_of_string lhs) with
           | Coupling.Vector ->
               begin
