@@ -1,6 +1,6 @@
-! WHIZARD 2.6.4 Aug 23 2018
+! WHIZARD 2.8.2 Oct 24 2019
 !
-! Copyright (C) 1999-2018 by
+! Copyright (C) 1999-2019 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
@@ -33,7 +33,6 @@ module sf_base
   use io_units
   use format_utils, only: write_separator
   use format_defs, only: FMT_17, FMT_19
-  use physics_defs, only: n_beam_structure_int
   use diagnostics
   use lorentz
   use quantum_numbers
@@ -43,11 +42,16 @@ module sf_base
   use beams
   use sf_aux
   use sf_mappings
+  use constants, only: one, two
+  use physics_defs, only: n_beams_rescaled
 
   implicit none
   private
 
   public :: sf_rescale_t
+  public :: sf_rescale_collinear_t
+  public :: sf_rescale_real_t
+  public :: sf_rescale_dglap_t
   public :: sf_data_t
   public :: sf_config_t
   public :: sf_int_t
@@ -69,18 +73,33 @@ module sf_base
 
 
   type, abstract :: sf_rescale_t
-    integer :: i_restricted_beam = -1
     integer :: i_beam = 0
-    logical :: gluon = .false.
   contains
     procedure (sf_rescale_apply), deferred :: apply
     procedure :: set_i_beam => sf_rescale_set_i_beam
-    procedure :: restrict_to_beam => sf_rescale_restrict_to_beam
-    procedure :: is_restricted => sf_rescale_is_restricted
-    procedure :: set_gluons => sf_rescale_set_gluons
-    procedure :: has_gluons => sf_rescale_has_gluons
   end type sf_rescale_t
 
+  type, extends (sf_rescale_t) :: sf_rescale_collinear_t
+     real(default) :: xi_tilde
+  contains
+    procedure :: apply => sf_rescale_collinear_apply
+    procedure :: set => sf_rescale_collinear_set
+  end type sf_rescale_collinear_t
+
+  type, extends (sf_rescale_t) :: sf_rescale_real_t
+     real(default) :: xi, y
+  contains
+    procedure :: apply => sf_rescale_real_apply
+    procedure :: set => sf_rescale_real_set
+  end type sf_rescale_real_t
+
+  type, extends(sf_rescale_t) :: sf_rescale_dglap_t
+     real(default), dimension(:), allocatable :: z
+   contains
+     procedure :: apply => sf_rescale_dglap_apply
+     procedure :: set => sf_rescale_dglap_set
+  end type sf_rescale_dglap_t
+
   type, abstract :: sf_data_t
    contains
      procedure (sf_data_write), deferred :: write
@@ -232,6 +251,7 @@ module sf_base
      procedure :: get_status => sf_chain_instance_get_status
      procedure :: get_matrix_elements => sf_chain_instance_get_matrix_elements
      procedure :: get_beam_int_ptr => sf_chain_instance_get_beam_int_ptr
+     procedure :: get_n_sub => sf_chain_instance_get_n_sub
   end type sf_chain_instance_t
 
 
@@ -315,8 +335,7 @@ module sf_base
   end interface
 
   abstract interface
-     subroutine sf_int_inverse_kinematics (sf_int, x, xb, f, r, rb, map, &
-          set_momenta)
+     subroutine sf_int_inverse_kinematics (sf_int, x, xb, f, r, rb, map, set_momenta)
        import
        class(sf_int_t), intent(inout) :: sf_int
        real(default), dimension(:), intent(in) :: x
@@ -330,13 +349,12 @@ module sf_base
   end interface
 
   abstract interface
-     subroutine sf_int_apply (sf_int, scale, rescale, i_sub, fill_sub)
+     subroutine sf_int_apply (sf_int, scale, rescale, i_sub)
        import
        class(sf_int_t), intent(inout) :: sf_int
        real(default), intent(in) :: scale
        class(sf_rescale_t), intent(in), optional :: rescale
        integer, intent(in), optional :: i_sub
-       logical, intent(in), optional :: fill_sub
      end subroutine sf_int_apply
   end interface
 
@@ -349,31 +367,73 @@ contains
     func%i_beam = i_beam
   end subroutine sf_rescale_set_i_beam
 
-  subroutine sf_rescale_restrict_to_beam (func, i_beam)
-    class(sf_rescale_t), intent(inout) :: func
-    integer, intent(in) :: i_beam
-    if (func%i_restricted_beam > 0) &
-         call msg_bug ("[sf_rescale_restrict_to_beam] restricted beam already set.")
-    func%i_restricted_beam = i_beam
-  end subroutine sf_rescale_restrict_to_beam
+  subroutine sf_rescale_collinear_apply (func, x)
+    class(sf_rescale_collinear_t), intent(in) :: func
+    real(default), intent(inout) :: x
+    real(default) :: xi
+    if (debug2_active (D_BEAMS)) then
+       print *, 'Rescaling function - Collinear: '
+       print *, 'Input, unscaled x: ', x
+       print *, 'xi_tilde: ', func%xi_tilde
+    end if
+    xi = func%xi_tilde * (one - x)
+    x = x / (one - xi)
+    if (debug2_active (D_BEAMS))  print *, 'rescaled x: ', x
+  end subroutine sf_rescale_collinear_apply
 
-  logical function sf_rescale_is_restricted (func, i_beam) result (yorn)
-    class(sf_rescale_t), intent(in) :: func
-    integer, intent(in) :: i_beam
-    yorn = (func%i_restricted_beam > 0)
-    yorn = yorn .and. (func%i_restricted_beam /= i_beam)
-  end function sf_rescale_is_restricted
+  subroutine sf_rescale_collinear_set (func, xi_tilde)
+    class(sf_rescale_collinear_t), intent(inout) :: func
+    real(default), intent(in) :: xi_tilde
+    func%xi_tilde = xi_tilde
+  end subroutine sf_rescale_collinear_set
 
-  subroutine sf_rescale_set_gluons (func, yorn)
-    class(sf_rescale_t), intent(inout) :: func
-    logical, intent(in) :: yorn
-    func%gluon = yorn
-  end subroutine sf_rescale_set_gluons
+  subroutine sf_rescale_real_apply (func, x)
+    class(sf_rescale_real_t), intent(in) :: func
+    real(default), intent(inout) :: x
+    real(default) :: onepy, onemy
+    if (debug2_active (D_BEAMS)) then
+       print *, 'Rescaling function - Real: '
+       print *, 'Input, unscaled: ', x
+       print *, 'Beam index: ', func%i_beam
+       print *, 'xi: ', func%xi, 'y: ', func%y
+    end if
+    x = x / sqrt (one - func%xi)
+    onepy = one + func%y; onemy = one - func%y
+    if (func%i_beam == 1) then
+       x = x * sqrt ((two - func%xi * onemy) / (two - func%xi * onepy))
+    else if (func%i_beam == 2) then
+       x = x * sqrt ((two - func%xi * onepy) / (two - func%xi * onemy))
+    else
+       call msg_fatal ("sf_rescale_real_apply - invalid beam index")
+    end if
+    if (debug2_active (D_BEAMS))  print *, 'rescaled x: ', x
+  end subroutine sf_rescale_real_apply
 
-  logical function sf_rescale_has_gluons (func) result (yorn)
-    class(sf_rescale_t), intent(in) :: func
-    yorn = func%gluon
-  end function sf_rescale_has_gluons
+  subroutine sf_rescale_real_set (func, xi, y)
+    class(sf_rescale_real_t), intent(inout) :: func
+    real(default), intent(in) :: xi, y
+    func%xi = xi; func%y = y
+  end subroutine sf_rescale_real_set
+
+  subroutine sf_rescale_dglap_apply (func, x)
+    class(sf_rescale_dglap_t), intent(in) :: func
+    real(default), intent(inout) :: x
+    if (debug2_active (D_BEAMS))  then
+       print *, "Rescaling function - DGLAP:"
+       print *, "Input: ", x
+       print *, "Beam index: ", func%i_beam
+       print *, "z: ", func%z
+    end if
+    x = x / func%z(func%i_beam)
+    if (debug2_active (D_BEAMS)) print *, "scaled x: ", x
+  end subroutine sf_rescale_dglap_apply
+
+  subroutine sf_rescale_dglap_set (func, z)
+    class(sf_rescale_dglap_t), intent(inout) :: func
+    real(default), dimension(:), intent(in) :: z
+    ! allocate-on-assginment
+    func%z = z
+  end subroutine sf_rescale_dglap_set
 
   function sf_data_is_generator (data) result (flag)
     class(sf_data_t), intent(in) :: data
@@ -1348,15 +1408,14 @@ contains
                 associate (sf => chain%sf(i))
                    if (yorn) then
                       if (int%get_n_sub () == 0) then
-                         call int%declare_subtraction (n_beam_structure_int)
+                         call int%declare_subtraction (n_beams_rescaled)
                       end if
                       if (sf%int%interaction_t%get_n_sub () == 0) then
-                         call sf%int%interaction_t%declare_subtraction &
-                              (n_beam_structure_int)
+                         call sf%int%interaction_t%declare_subtraction (n_beams_rescaled)
                       end if
                    end if
                    call sf%eval%init_product (int, sf%int%interaction_t, mask,&
-                        & ignore_sub = .true.)
+                        & ignore_sub_for_qn = .true.)
                    if (sf%eval%is_empty ()) then
                       chain%status = SF_FAILED_CONNECTIONS
                       return
@@ -1553,7 +1612,8 @@ contains
                   end do
                   do c = 1, size (sf%f)
                      call sf%int%inverse_kinematics &
-                          (sf%x, sf%xb, sf%f(c), sf%r(:,c), sf%rb(:,c), sf%m(c), c==1)
+                          (sf%x, sf%xb, sf%f(c), sf%r(:,c), sf%rb(:,c), sf%m(c), &
+                          set_momenta = c==1)
                      chain%f(c) = chain%f(c) * sf%f(c)
                      do j = 1, size (sf%x)
                         chain%r (sf%int%par_index(j),c) = sf%r (j,c)
@@ -1613,7 +1673,8 @@ contains
                   end do
                   do c = 1, size (sf%f)
                      call sf%int%inverse_kinematics &
-                          (sf%x, sf%xb, sf%f(c), sf%r(:,c), sf%rb(:,c), sf%m(c), c==1)
+                          (sf%x, sf%xb, sf%f(c), sf%r(:,c), sf%rb(:,c), sf%m(c), &
+                          set_momenta = .false.)
                      chain%f(c) = chain%f(c) * sf%f(c)
                      do j = 1, size (sf%x)
                         chain%r (sf%int%par_index(j),c) = sf%r (j,c)
@@ -1657,44 +1718,43 @@ contains
     type(interaction_t), pointer :: out_int
     real(default) :: sf_sum
     integer :: i_beam, i_sub, n_sub
+    logical :: rescale
+    n_sub = 0
+    rescale = .false.; if (present (sf_rescale)) rescale = .true.
+    if (rescale) then
+       n_sub = chain%get_n_sub ()
+    end if
     if (chain%status >= SF_DONE_KINEMATICS) then
        if (allocated (chain%sf)) then
           if (size (chain%sf) /= 0) then
              do i_beam = 1, size (chain%sf)
                 associate (sf => chain%sf(i_beam))
-                  n_sub = 0 ! default: no looping over rescaled beams
-                  if (present (sf_rescale)) then
-                     ! TODO sbrass cache n_sub as it is computed from the state matrix
-                     n_sub = sf%int%get_n_sub ()
-                     call sf_rescale%set_i_beam (i_beam)
-                  end if
-                  SUB: do i_sub = 0, n_sub
-                     select case (i_sub)
-                     case (0)
-                        if (n_sub == 0) then
-                           call sf%int%apply (scale, sf_rescale)
-                        else
-                           call sf%int%apply (scale, fill_sub = .true.)
-                        end if
-                     case (1:2)
-                        if (present (sf_rescale)) then
-                           if (sf_rescale%is_restricted (i_beam)) cycle SUB
-                        end if
-                        if (i_sub == i_beam) then
-                           call sf%int%apply(scale, sf_rescale, i_sub)
-                        end if
-                     case (3:4)
-                        ! dummy : handled more appropriately on a lower level (sf%int%apply ())
-                     case default
-                        call msg_bug ("sf_chain_instance_evaluate: more than 2&
-                             & subtraction indices are curently not handled.")
-                     end select
-                     if (sf%int%status <= SF_FAILED_EVALUATION) then
-                        chain%status = SF_FAILED_EVALUATION
-                        return
-                     end if
-                  end do SUB
-                  if (.not. sf%eval%is_empty ())  call sf%eval%evaluate ()
+                   if (rescale) then
+                      call sf_rescale%set_i_beam (i_beam)
+                      do i_sub = 0, n_sub
+                         select case (i_sub)
+                         case (0)
+                            if (n_sub == 0) then
+                               call sf%int%apply (scale, sf_rescale, i_sub = i_sub)
+                            else
+                               call sf%int%apply (scale, i_sub = i_sub)
+                            end if
+                         case default
+                            if (i_beam == i_sub) then
+                               call sf%int%apply (scale, sf_rescale, i_sub = i_sub)
+                            else
+                               call sf%int%apply (scale, i_sub = i_sub)
+                            end if
+                         end select
+                      end do
+                   else
+                      call sf%int%apply (scale, i_sub = n_sub)
+                   end if
+                   if (sf%int%status <= SF_FAILED_EVALUATION) then
+                      chain%status = SF_FAILED_EVALUATION
+                      return
+                   end if
+                   if (.not. sf%eval%is_empty ())  call sf%eval%evaluate ()
                 end associate
              end do
              out_int => chain%get_out_int_ptr ()
@@ -1792,5 +1852,12 @@ contains
     int => beam_get_int_ptr (chain%beam_t)
   end function sf_chain_instance_get_beam_int_ptr
 
+  integer function sf_chain_instance_get_n_sub (chain) result (n_sub)
+    type(interaction_t), pointer :: int
+    class(sf_chain_instance_t), intent(in), target :: chain
+    int => beam_get_int_ptr (chain%beam_t)
+    n_sub = int%get_n_sub ()
+  end function sf_chain_instance_get_n_sub
+
 
 end module sf_base
