@@ -1,6 +1,6 @@
-! WHIZARD 2.6.4 Aug 23 2018
+! WHIZARD 2.8.2 Oct 24 2019
 !
-! Copyright (C) 1999-2018 by
+! Copyright (C) 1999-2019 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
@@ -2332,12 +2332,11 @@ contains
     end if
   end subroutine sf_test_inverse_kinematics
 
-  subroutine sf_test_apply (sf_int, scale, rescale, i_sub, fill_sub)
+  subroutine sf_test_apply (sf_int, scale, rescale, i_sub)
     class(sf_test_t), intent(inout) :: sf_int
     real(default), intent(in) :: scale
     class(sf_rescale_t), intent(in), optional :: rescale
     integer, intent(in), optional :: i_sub
-    logical, intent(in), optional :: fill_sub
     select case (sf_int%data%mode)
     case (0)
        call sf_int%set_matrix_element &
@@ -2532,12 +2531,11 @@ contains
     end if
   end subroutine sf_test_spectrum_inverse_kinematics
 
-  subroutine sf_test_spectrum_apply (sf_int, scale, rescale, i_sub, fill_sub)
+  subroutine sf_test_spectrum_apply (sf_int, scale, rescale, i_sub)
     class(sf_test_spectrum_t), intent(inout) :: sf_int
     real(default), intent(in) :: scale
     class(sf_rescale_t), intent(in), optional :: rescale
     integer, intent(in), optional :: i_sub
-    logical, intent(in), optional :: fill_sub
     call sf_int%set_matrix_element &
          (cmplx (1._default, kind=default))
     sf_int%status = SF_EVALUATED
@@ -2701,12 +2699,11 @@ contains
     if (set_mom)  call sf_int%reduce_momenta (x)
   end subroutine sf_test_generator_inverse_kinematics
 
-  subroutine sf_test_generator_apply (sf_int, scale, rescale, i_sub, fill_sub)
+  subroutine sf_test_generator_apply (sf_int, scale, rescale, i_sub)
     class(sf_test_generator_t), intent(inout) :: sf_int
     real(default), intent(in) :: scale
     class(sf_rescale_t), intent(in), optional :: rescale
     integer, intent(in), optional :: i_sub
-    logical, intent(in), optional :: fill_sub
     call sf_int%set_matrix_element &
          (cmplx (1._default, kind=default))
     sf_int%status = SF_EVALUATED
