@@ -1,6 +1,6 @@
-! WHIZARD 2.6.4 Aug 23 2018
+! WHIZARD 2.8.2 Oct 24 2019
 !
-! Copyright (C) 1999-2018 by
+! Copyright (C) 1999-2019 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
@@ -454,12 +454,11 @@ contains
     end if
   end subroutine circe2_inverse_kinematics
 
-  subroutine circe2_apply (sf_int, scale, rescale, i_sub, fill_sub)
+  subroutine circe2_apply (sf_int, scale, rescale, i_sub)
     class(circe2_t), intent(inout) :: sf_int
     real(default), intent(in) :: scale
     class(sf_rescale_t), intent(in), optional :: rescale
     integer, intent(in), optional :: i_sub
-    logical, intent(in), optional :: fill_sub
     complex(default) :: f
     associate (data => sf_int%data)
       f = 1
