@@ -1,6 +1,6 @@
-! WHIZARD 2.6.4 Aug 23 2018
+! WHIZARD 2.8.2 Oct 24 2019
 !
-! Copyright (C) 1999-2018 by
+! Copyright (C) 1999-2019 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
@@ -100,8 +100,8 @@ module sf_epa
      procedure :: init => epa_init
      procedure :: setup_constants => epa_setup_constants
      procedure :: complete_kinematics => epa_complete_kinematics
-     procedure :: inverse_kinematics => epa_inverse_kinematics
      procedure :: recover_x => sf_epa_recover_x
+     procedure :: inverse_kinematics => epa_inverse_kinematics
      procedure :: apply => epa_apply
   end type epa_t
 
@@ -384,6 +384,16 @@ contains
     end select
   end subroutine epa_complete_kinematics
 
+  subroutine sf_epa_recover_x (sf_int, x, xb, x_free)
+    class(epa_t), intent(inout) :: sf_int
+    real(default), dimension(:), intent(out) :: x
+    real(default), dimension(:), intent(out) :: xb
+    real(default), intent(inout), optional :: x_free
+    call sf_int%base_recover_x (x, xb, x_free)
+    sf_int%x  = x(1)
+    sf_int%xb = xb(1)
+  end subroutine sf_epa_recover_x
+
   subroutine epa_inverse_kinematics (sf_int, x, xb, f, r, rb, map, set_momenta)
     class(epa_t), intent(inout) :: sf_int
     real(default), dimension(:), intent(in) :: x
@@ -422,33 +432,17 @@ contains
     if (set_mom) then
        call sf_int%split_momentum (x, xb)
        select case (sf_int%status)
-       case (SF_DONE_KINEMATICS)
-          sf_int%x  = x(1)
-          sf_int%xb = xb(1)
-          sf_int%E  = energy (sf_int%get_momentum (1))
-       case (SF_FAILED_KINEMATICS)
-          sf_int%x = 0
-          f = 0
+       case (SF_FAILED_KINEMATICS);  f = 0
        end select
     end if
+    sf_int%E  = energy (sf_int%get_momentum (1))
   end subroutine epa_inverse_kinematics
 
-  subroutine sf_epa_recover_x (sf_int, x, xb, x_free)
-    class(epa_t), intent(inout) :: sf_int
-    real(default), dimension(:), intent(out) :: x
-    real(default), dimension(:), intent(out) :: xb
-    real(default), intent(inout), optional :: x_free
-    call sf_int%base_recover_x (x, xb, x_free)
-    sf_int%x  = x(1)
-    sf_int%xb = xb(1)
-  end subroutine sf_epa_recover_x
-
-  subroutine epa_apply (sf_int, scale, rescale, i_sub, fill_sub)
+  subroutine epa_apply (sf_int, scale, rescale, i_sub)
     class(epa_t), intent(inout) :: sf_int
     real(default), intent(in) :: scale
     class(sf_rescale_t), intent(in), optional :: rescale
     integer, intent(in), optional :: i_sub
-    logical, intent(in), optional :: fill_sub
     real(default) :: x, xb, qminsq, qmaxsq, f, E
     associate (data => sf_int%data)
       x = sf_int%x
