@@ -1,6 +1,6 @@
-! WHIZARD 2.6.4 Aug 23 2018
+! WHIZARD 2.8.2 Oct 24 2019
 !
-! Copyright (C) 1999-2018 by
+! Copyright (C) 1999-2019 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
@@ -111,6 +111,7 @@ module sf_ewa
      procedure :: init => ewa_init
      procedure :: setup_constants => ewa_setup_constants
      procedure :: complete_kinematics => ewa_complete_kinematics
+     procedure :: recover_x => sf_ewa_recover_x
      procedure :: inverse_kinematics => ewa_inverse_kinematics
      procedure :: apply => ewa_apply
   end type ewa_t
@@ -553,6 +554,16 @@ contains
     end select
   end subroutine ewa_complete_kinematics
 
+  subroutine sf_ewa_recover_x (sf_int, x, xb, x_free)
+    class(ewa_t), intent(inout) :: sf_int
+    real(default), dimension(:), intent(out) :: x
+    real(default), dimension(:), intent(out) :: xb
+    real(default), intent(inout), optional :: x_free
+    call sf_int%base_recover_x (x, xb, x_free)
+    sf_int%x  = x(1)
+    sf_int%xb = xb(1)
+  end subroutine sf_ewa_recover_x
+
   subroutine ewa_inverse_kinematics (sf_int, x, xb, f, r, rb, map, set_momenta)
     class(ewa_t), intent(inout) :: sf_int
     real(default), dimension(:), intent(in) :: x
@@ -600,23 +611,16 @@ contains
     if (set_mom) then
        call sf_int%split_momentum (x, xb)
        select case (sf_int%status)
-       case (SF_DONE_KINEMATICS)
-          sf_int%x = x(1)
-          sf_int%xb= xb(1)
-       case (SF_FAILED_KINEMATICS)
-          sf_int%x = 0
-          sf_int%xb= 0
-          f = 0
+       case (SF_FAILED_KINEMATICS);  f = 0
        end select
     end if
   end subroutine ewa_inverse_kinematics
 
-  subroutine ewa_apply (sf_int, scale, rescale, i_sub, fill_sub)
+  subroutine ewa_apply (sf_int, scale, rescale, i_sub)
     class(ewa_t), intent(inout) :: sf_int
     real(default), intent(in) :: scale
     class(sf_rescale_t), intent(in), optional :: rescale
     integer, intent(in), optional :: i_sub
-    logical, intent(in), optional :: fill_sub
     real(default) :: x, xb, pt2, c1, c2
     real(default) :: cv, ca
     real(default) :: f, fm, fp, fL
