@@ -1,6 +1,6 @@
-! WHIZARD 2.6.4 Aug 23 2018
+! WHIZARD 2.8.2 Oct 24 2019
 !
-! Copyright (C) 1999-2018 by
+! Copyright (C) 1999-2019 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
@@ -36,7 +36,6 @@ module sf_lhapdf
   use system_dependencies, only: LHAPDF5_AVAILABLE
   use system_dependencies, only: LHAPDF6_AVAILABLE
   use diagnostics
-  use physics_defs, only: n_beam_gluon_offset
   use physics_defs, only: PROTON, PHOTON, PIPLUS, GLUON
   use physics_defs, only: HADRON_REMNANT_SINGLET
   use physics_defs, only: HADRON_REMNANT_TRIPLET
@@ -102,6 +101,7 @@ module sf_lhapdf
      real(default) :: s = 0
    contains
      procedure :: complete_kinematics => lhapdf_complete_kinematics
+     procedure :: recover_x => lhapdf_recover_x
      procedure :: inverse_kinematics => lhapdf_inverse_kinematics
      procedure :: type_string => lhapdf_type_string
      procedure :: write => lhapdf_write
@@ -341,6 +341,15 @@ contains
     end select
   end subroutine lhapdf_complete_kinematics
 
+  subroutine lhapdf_recover_x (sf_int, x, xb, x_free)
+    class(lhapdf_t), intent(inout) :: sf_int
+    real(default), dimension(:), intent(out) :: x
+    real(default), dimension(:), intent(out) :: xb
+    real(default), intent(inout), optional :: x_free
+    call sf_int%base_recover_x (x, xb, x_free)
+    sf_int%x  = x(1)
+  end subroutine lhapdf_recover_x
+
   subroutine lhapdf_inverse_kinematics (sf_int, x, xb, f, r, rb, map, set_momenta)
     class(lhapdf_t), intent(inout) :: sf_int
     real(default), dimension(:), intent(in) :: x
@@ -362,11 +371,7 @@ contains
     if (set_mom) then
        call sf_int%split_momentum (x, xb)
        select case (sf_int%status)
-       case (SF_DONE_KINEMATICS)
-          sf_int%x = x(1)
-       case (SF_FAILED_KINEMATICS)
-          sf_int%x = 0
-          f = 0
+       case (SF_FAILED_KINEMATICS);  f = 0
        end select
     end if
   end subroutine lhapdf_inverse_kinematics
@@ -424,8 +429,8 @@ contains
     end if
     if (present (hoppet_b_matching))  data%hoppet_b_matching = hoppet_b_matching
     if (LHAPDF5_AVAILABLE) then
-       call lhapdf_initialize &
-            (data%set, data%prefix, data%file, data%member, &
+       call lhapdf_initialize (data%set, &
+            data%prefix, data%file, data%member, &
             b_match = data%hoppet_b_matching)
        call GetXminM (data%set, data%member, xmin)
        call GetXmaxM (data%set, data%member, xmax)
@@ -437,8 +442,8 @@ contains
        data%qmax = sqrt (q2max)
        data%has_photon = has_photon ()
     else if (LHAPDF6_AVAILABLE) then
-       call lhapdf_initialize &
-            (data%set, data%prefix, data%file, data%member, &
+       call lhapdf_initialize (data%set, &
+            data%prefix, data%file, data%member, &
             data%pdf, data%hoppet_b_matching)
        data%xmin = data%pdf%getxmin ()
        data%xmax = data%pdf%getxmax ()
@@ -602,30 +607,19 @@ contains
     end select
   end subroutine lhapdf_init
 
-  subroutine lhapdf_apply (sf_int, scale, rescale, i_sub, fill_sub)
+  subroutine lhapdf_apply (sf_int, scale, rescale, i_sub)
     class(lhapdf_t), intent(inout) :: sf_int
     real(default), intent(in) :: scale
     class(sf_rescale_t), intent(in), optional :: rescale
     integer, intent(in), optional :: i_sub
-    logical, intent(in), optional :: fill_sub
     real(default) :: x, s
     double precision :: xx, qq, ss
     double precision, dimension(-6:6) :: ff
     double precision :: fphot
     complex(default), dimension(:), allocatable :: fc
     integer :: i, i_sub_opt, j_sub
-    logical :: fill_sub_opt
     external :: evolvePDFM, evolvePDFpM
     i_sub_opt = 0; if (present (i_sub)) i_sub_opt = i_sub
-    fill_sub_opt = .false.; if (present (fill_sub)) fill_sub_opt = fill_sub
-    if (present (rescale) .and. fill_sub_opt) then
-       call msg_bug ("[lhapdf_apply] &
-            & sf_rescale and fill_sub option are mutually exclusive.")
-    end if
-    if (i_sub_opt > 0 .and. fill_sub_opt) then
-       call msg_bug ("[lhapdf_apply] &
-            & i_sub and fill_sub options are mutually exclusive.")
-    end if
     associate (data => sf_int%data)
       sf_int%q = scale
       x = sf_int%x
@@ -636,17 +630,16 @@ contains
          call msg_debug2 (D_BEAMS, "lhapdf_apply")
          call msg_debug2 (D_BEAMS, "rescale: ", present(rescale))
          call msg_debug2 (D_BEAMS, "i_sub: ", i_sub_opt)
-         call msg_debug2 (D_BEAMS, "fill_sub: ", fill_sub_opt)
          call msg_debug2 (D_BEAMS, "x: ", x)
       end if
       qq = min (data%qmax, scale)
       qq = max (data%qmin, qq)
-      if (.not. data% photon) then
+      if (.not. data%photon) then
          if (data%invert) then
             if (data%has_photon) then
                if (LHAPDF5_AVAILABLE) then
                   call evolvePDFphotonM &
-                       (data% set, xx, qq, ff(6:-6:-1), fphot)
+                       (data%set, xx, qq, ff(6:-6:-1), fphot)
                else if (LHAPDF6_AVAILABLE) then
                   call data%pdf%evolve_pdfphotonm &
                        (xx, qq, ff(6:-6:-1), fphot)
@@ -656,7 +649,7 @@ contains
                   call hoppet_eval (xx, qq, ff(6:-6:-1))
                else
                   if (LHAPDF5_AVAILABLE) then
-                     call evolvePDFM (data% set, xx, qq, ff(6:-6:-1))
+                     call evolvePDFM (data%set, xx, qq, ff(6:-6:-1))
                   else if (LHAPDF6_AVAILABLE) then
                      call data%pdf%evolve_pdfm (xx, qq, ff(6:-6:-1))
                   end if
@@ -665,7 +658,7 @@ contains
          else
             if (data%has_photon) then
                if (LHAPDF5_AVAILABLE) then
-                  call evolvePDFphotonM (data% set, xx, qq, ff, fphot)
+                  call evolvePDFphotonM (data%set, xx, qq, ff, fphot)
                else if (LHAPDF6_AVAILABLE) then
                   call data%pdf%evolve_pdfphotonm (xx, qq, ff, fphot)
                end if
@@ -674,7 +667,7 @@ contains
                   call hoppet_eval (xx, qq, ff)
                else
                   if (LHAPDF5_AVAILABLE) then
-                     call evolvePDFM (data% set, xx, qq, ff)
+                     call evolvePDFM (data%set, xx, qq, ff)
                   else if (LHAPDF6_AVAILABLE) then
                      call data%pdf%evolve_pdfm (xx, qq, ff)
                   end if
@@ -684,8 +677,8 @@ contains
       else
          ss = s
          if (LHAPDF5_AVAILABLE) then
-            call evolvePDFpM (data% set, xx, qq, &
-                 ss, data% photon_scheme, ff)
+            call evolvePDFpM (data%set, xx, qq, &
+                 ss, data%photon_scheme, ff)
          else if (LHAPDF6_AVAILABLE) then
             call data%pdf%evolve_pdfpm (xx, qq, ss, &
                  data%photon_scheme, ff)
@@ -694,28 +687,14 @@ contains
       if (data%has_photon) then
          allocate (fc (count ([data%mask, data%mask_photon])))
          fc = max (pack ([ff, fphot] / x, &
-              [data% mask, data%mask_photon]), 0._default)
+              [data%mask, data%mask_photon]), 0._default)
       else
          allocate (fc (count (data%mask)))
          fc = max (pack (ff / x, data%mask), 0._default)
       end if
     end associate
     if (debug_active (D_BEAMS)) print *, 'Set pdfs: ', real (fc)
-    if (present (rescale) .and. i_sub_opt > 0) then
-       call sf_int%set_matrix_element (fc, [(i_sub_opt * size(fc) + i, i = 1, size(fc))])
-       if (rescale%has_gluons ()) then
-          j_sub = i_sub_opt + n_beam_gluon_offset
-          call sf_int%set_matrix_element (&
-               spread (fc(7), 1, size(fc)), [(j_sub * size(fc) + i, i = 1, size(fc))])
-       end if
-    else
-       call sf_int%set_matrix_element (fc, [(i, i = 1, size(fc))])
-    end if
-    if(fill_sub_opt) then
-       do j_sub = 1, sf_int%get_n_sub ()
-          call sf_int%set_matrix_element (fc, [(j_sub * size(fc) + i, i = 1, size(fc))])
-       end do
-    end if
+    call sf_int%set_matrix_element (fc, [(i_sub_opt * size(fc) + i, i = 1, size(fc))])
     sf_int%status = SF_EVALUATED
   end subroutine lhapdf_apply
 
