@@ -1,6 +1,6 @@
-! WHIZARD 2.6.4 Aug 23 2018
+! WHIZARD 2.8.2 Oct 24 2019
 !
-! Copyright (C) 1999-2018 by
+! Copyright (C) 1999-2019 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
@@ -35,7 +35,6 @@ module sf_pdf_builtin
   use format_defs, only: FMT_17
   use diagnostics
   use os_interface
-  use physics_defs, only: n_beam_gluon_offset
   use physics_defs, only: PROTON, PHOTON, GLUON
   use physics_defs, only: HADRON_REMNANT_SINGLET
   use physics_defs, only: HADRON_REMNANT_TRIPLET
@@ -91,6 +90,7 @@ module sf_pdf_builtin
     procedure :: write => pdf_builtin_write
     procedure :: init => pdf_builtin_init
     procedure :: complete_kinematics => pdf_builtin_complete_kinematics
+    procedure :: recover_x => pdf_builtin_recover_x
     procedure :: inverse_kinematics => pdf_builtin_inverse_kinematics
     procedure :: apply => pdf_builtin_apply
   end type pdf_builtin_t
@@ -327,6 +327,15 @@ contains
     end select
   end subroutine pdf_builtin_complete_kinematics
 
+  subroutine pdf_builtin_recover_x (sf_int, x, xb, x_free)
+    class(pdf_builtin_t), intent(inout) :: sf_int
+    real(default), dimension(:), intent(out) :: x
+    real(default), dimension(:), intent(out) :: xb
+    real(default), intent(inout), optional :: x_free
+    call sf_int%base_recover_x (x, xb, x_free)
+    sf_int%x  = x(1)
+  end subroutine pdf_builtin_recover_x
+
   subroutine pdf_builtin_inverse_kinematics (sf_int, x, xb, f, r, rb, map, set_momenta)
     class(pdf_builtin_t), intent(inout) :: sf_int
     real(default), dimension(:), intent(in) :: x
@@ -348,38 +357,23 @@ contains
     if (set_mom) then
        call sf_int%split_momentum (x, xb)
        select case (sf_int%status)
-       case (SF_DONE_KINEMATICS)
-          sf_int%x = x(1)
-       case (SF_FAILED_KINEMATICS)
-          sf_int%x = 0
-          f = 0
+       case (SF_FAILED_KINEMATICS);  f = 0
        end select
     end if
   end subroutine pdf_builtin_inverse_kinematics
 
-  subroutine pdf_builtin_apply (sf_int, scale, rescale, i_sub, fill_sub)
+  subroutine pdf_builtin_apply (sf_int, scale, rescale, i_sub)
     class(pdf_builtin_t), intent(inout) :: sf_int
     real(default), intent(in) :: scale
     class(sf_rescale_t), intent(in), optional :: rescale
     integer, intent(in), optional :: i_sub
-    logical, intent(in), optional :: fill_sub
     real(default), dimension(-6:6) :: ff
     real(double), dimension(-6:6) :: ff_dbl
     real(default) :: x, fph
     real(double) :: xx, qq
     complex(default), dimension(:), allocatable :: fc
     integer :: i, j_sub, i_sub_opt
-    logical :: fill_sub_opt
     i_sub_opt = 0; if (present (i_sub)) i_sub_opt = i_sub
-    fill_sub_opt = .false.; if (present (fill_sub)) fill_sub_opt = fill_sub
-    if (present (rescale) .and. fill_sub_opt) then
-       call msg_bug ("[pdf_builtin_apply] &
-            & sf_rescale and fill_sub option are mutually exclusive.")
-    end if
-    if (i_sub_opt > 0 .and. fill_sub_opt) then
-       call msg_bug ("[pdf_builtin_apply] &
-            & i_sub and fill_sub options are mutually exclusive.")
-    end if
     associate (data => sf_int%data)
       sf_int%q = scale
       x = sf_int%x
@@ -388,7 +382,6 @@ contains
          call msg_debug2 (D_BEAMS, "pdf_builtin_apply")
          call msg_debug2 (D_BEAMS, "rescale: ", present(rescale))
          call msg_debug2 (D_BEAMS, "i_sub: ", i_sub_opt)
-         call msg_debug2 (D_BEAMS, "fill_sub: ", fill_sub_opt)
          call msg_debug2 (D_BEAMS, "x: ", x)
       end if
       xx = x
@@ -426,21 +419,7 @@ contains
       end if
     end associate
     if (debug_active (D_BEAMS)) print *, 'Set pdfs: ', real (fc)
-    if (present (rescale) .and. i_sub_opt > 0) then
-       call sf_int%set_matrix_element (fc, [(i_sub_opt * size(fc) + i, i = 1, size(fc))])
-       if (rescale%has_gluons ()) then
-          j_sub = i_sub_opt + n_beam_gluon_offset
-          call sf_int%set_matrix_element (&
-               spread (fc(7), 1, size(fc)), [(j_sub * size(fc) + i, i = 1, size(fc))])
-       end if
-    else
-       call sf_int%set_matrix_element (fc, [(i, i = 1, size(fc))])
-    end if
-    if(fill_sub_opt) then
-       do j_sub = 1, sf_int%get_n_sub ()
-          call sf_int%set_matrix_element (fc, [(j_sub * size(fc) + i, i = 1, size(fc))])
-       end do
-    end if
+    call sf_int%set_matrix_element (fc, [(i_sub_opt * size(fc) + i, i = 1, size(fc))])
     sf_int%status = SF_EVALUATED
   end subroutine pdf_builtin_apply
 
