@@ -1,6 +1,6 @@
-! WHIZARD 2.6.4 Aug 23 2018
+! WHIZARD 2.8.2 Oct 24 2019
 !
-! Copyright (C) 1999-2018 by
+! Copyright (C) 1999-2019 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
@@ -69,7 +69,7 @@ contains
     write (u, "(A)")  "* Create empty data object"
     write (u, "(A)")
 
-    call os_data_init (os_data)
+    call os_data%init ()
 
     call model%init_sm_test ()
     pdg_in = PROTON
@@ -128,7 +128,7 @@ contains
     write (u, "(A)")  "* Initialize configuration data"
     write (u, "(A)")
 
-    call os_data_init (os_data)
+    call os_data%init ()
     call model%init_sm_test ()
     call flv%init (PROTON, model)
     pdg_in = PROTON
@@ -232,7 +232,7 @@ contains
     write (u, "(A)")  "* Initialize configuration data"
     write (u, "(A)")
 
-    call os_data_init (os_data)
+    call os_data%init ()
 
     name = "CTEQ6L"
 
