@@ -1,6 +1,6 @@
-! WHIZARD 2.6.4 Aug 23 2018
+! WHIZARD 2.8.2 Oct 24 2019
 !
-! Copyright (C) 1999-2018 by
+! Copyright (C) 1999-2019 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
@@ -28,6 +28,7 @@
 
 module eio_hepmc
 
+  use kinds, only: default
   use iso_varying_string, string_t => varying_string
   use io_units
   use string_utils
@@ -54,6 +55,7 @@ module eio_hepmc
      logical :: use_alphas_from_file = .false.
      logical :: use_scale_from_file = .false.
      logical :: output_cross_section = .false.
+     logical :: hepmc2_mode = .false.
      type(hepmc_iostream_t) :: iostream
      type(hepmc_event_t) :: hepmc_event
      integer, dimension(:), allocatable :: proc_num_id
@@ -78,13 +80,14 @@ contains
   subroutine eio_hepmc_set_parameters &
        (eio, &
        recover_beams, use_alphas_from_file, use_scale_from_file, &
-       extension, output_cross_section)
+       extension, output_cross_section, hepmc2_mode)
     class(eio_hepmc_t), intent(inout) :: eio
     logical, intent(in), optional :: recover_beams
     logical, intent(in), optional :: use_alphas_from_file
     logical, intent(in), optional :: use_scale_from_file
     logical, intent(in), optional :: output_cross_section
     type(string_t), intent(in), optional :: extension
+    logical, intent(in), optional :: hepmc2_mode
     if (present (recover_beams)) &
          eio%recover_beams = recover_beams
     if (present (use_alphas_from_file)) &
@@ -98,6 +101,8 @@ contains
     end if
     if (present (output_cross_section)) &
          eio%output_cross_section = output_cross_section
+    if (present (hepmc2_mode)) &
+         eio%hepmc2_mode = hepmc2_mode
   end subroutine eio_hepmc_set_parameters
 
   subroutine eio_hepmc_write (object, unit)
@@ -120,6 +125,7 @@ contains
          object%use_scale_from_file
     write (u, "(3x,A,A,A)")     "File extension    = '", &
          char (object%extension), "'"
+    write (u, "(3x,A,L1)")    "HepMC2 compat.    = ", object%hepmc2_mode
     if (allocated (object%proc_num_id)) then
        write (u, "(3x,A)")  "Numerical process IDs:"
        do i = 1, size (object%proc_num_id)
@@ -135,13 +141,13 @@ contains
        write (msg_buffer, "(A,A,A)")  "Events: closing HepMC file '", &
             char (object%filename), "'"
        call msg_message ()
-       call hepmc_iostream_close (object%iostream)
+       call hepmc_iostream_close (object%iostream, object%hepmc2_mode)
        object%writing = .false.
     else if (object%reading) then
        write (msg_buffer, "(A,A,A)")  "Events: closing HepMC file '", &
             char (object%filename), "'"
        call msg_message ()
-       call hepmc_iostream_close (object%iostream)
+       call hepmc_iostream_close (object%iostream, object%hepmc2_mode)
        object%reading = .false.
     end if
   end subroutine eio_hepmc_final
@@ -154,8 +160,9 @@ contains
        write (msg_buffer, "(A,A,A)")  "Events: writing to HepMC file '", &
             char (eio%filename), "'"
        call msg_message ()
-       call hepmc_iostream_close (eio%iostream)
-       call hepmc_iostream_open_out (eio%iostream, eio%filename)
+       call hepmc_iostream_close (eio%iostream, eio%hepmc2_mode)
+       call hepmc_iostream_open_out (eio%iostream, &
+            eio%filename, eio%hepmc2_mode)
     end if
   end subroutine eio_hepmc_split_out
 
@@ -197,7 +204,8 @@ contains
          char (eio%filename), "'"
     call msg_message ()
     eio%writing = .true.
-    call hepmc_iostream_open_out (eio%iostream, eio%filename)
+    call hepmc_iostream_open_out (eio%iostream, &
+         eio%filename, eio%hepmc2_mode)
     if (present (success))  success = .true.
   end subroutine eio_hepmc_init_out
 
@@ -216,7 +224,8 @@ contains
     inquire (file = char (eio%filename), exist = exist)
     if (.not. exist)  call msg_fatal ("Events: HepMC file not found.")
     eio%reading = .true.
-    call hepmc_iostream_open_in (eio%iostream, eio%filename)
+    call hepmc_iostream_open_in (eio%iostream, &
+         eio%filename, eio%hepmc2_mode)
     if (present (success))  success = .true.
   end subroutine eio_hepmc_init_in
 
@@ -249,9 +258,11 @@ contains
        end if
        call hepmc_event_set_scale (eio%hepmc_event, event%get_fac_scale ())
        call hepmc_event_set_alpha_qcd (eio%hepmc_event, event%get_alpha_s ())
+       call hepmc_event_set_alpha_qed (eio%hepmc_event, -1._default)
        if (.not. eio%data%unweighted) &
           call hepmc_event_add_weight (eio%hepmc_event, event%weight_prc)
-       call hepmc_iostream_write_event (eio%iostream, eio%hepmc_event)
+       call hepmc_iostream_write_event (eio%iostream, &
+            eio%hepmc_event, eio%hepmc2_mode)
        call hepmc_event_final (eio%hepmc_event)
     else
        call eio%write ()
@@ -267,7 +278,8 @@ contains
     integer :: i, proc_num_id
     iostat = 0
     call hepmc_event_init (eio%hepmc_event)
-    call hepmc_iostream_read_event (eio%iostream, eio%hepmc_event, ok)
+    call hepmc_iostream_read_event (eio%iostream, &
+         eio%hepmc_event, ok=ok, hepmc2=eio%hepmc2_mode)
     proc_num_id = hepmc_event_get_process_id (eio%hepmc_event)
     if (.not. ok) then
        iostat = -1
