@@ -1,6 +1,6 @@
-! WHIZARD 2.6.4 Aug 23 2018
+! WHIZARD 2.8.2 Oct 24 2019
 !
-! Copyright (C) 1999-2018 by
+! Copyright (C) 1999-2019 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
@@ -28,6 +28,8 @@
 
 module eio_hepmc_ut
   use unit_tests
+  use system_dependencies, only: HEPMC2_AVAILABLE
+  use system_dependencies, only: HEPMC3_AVAILABLE
   use eio_hepmc_uti
 
   implicit none
@@ -40,12 +42,24 @@ contains
   subroutine eio_hepmc_test (u, results)
     integer, intent(in) :: u
     type(test_results_t), intent(inout) :: results
-    call test (eio_hepmc_1, "eio_hepmc_1", &
-         "write event contents", &
-         u, results)
-    call test (eio_hepmc_2, "eio_hepmc_2", &
-         "read event contents", &
-         u, results)
+    if (HEPMC2_AVAILABLE) then
+       call test (eio_hepmc_1, "eio_hepmc2_1", &
+            "write event contents", &
+            u, results)
+    else if (HEPMC3_AVAILABLE) then
+       call test (eio_hepmc_1, "eio_hepmc3_1", &
+            "write event contents", &
+            u, results)
+    end if
+    if (HEPMC2_AVAILABLE) then
+       call test (eio_hepmc_2, "eio_hepmc2_2", &
+            "read event contents", &
+            u, results)
+    else if (HEPMC3_AVAILABLE) then
+       call test (eio_hepmc_2, "eio_hepmc3_2", &
+            "read event contents", &
+            u, results)
+    end if
   end subroutine eio_hepmc_test
 
 
