@@ -1,6 +1,6 @@
-! WHIZARD 2.6.4 Aug 23 2018
+! WHIZARD 2.8.2 Oct 24 2019
 !
-! Copyright (C) 1999-2018 by
+! Copyright (C) 1999-2019 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
@@ -30,7 +30,10 @@ module eio_hepmc_uti
 
   use kinds, only: default
   use iso_varying_string, string_t => varying_string
+  use system_dependencies, only: HEPMC2_AVAILABLE
+  use system_dependencies, only: HEPMC3_AVAILABLE
   use io_units
+  use diagnostics
   use model_data
   use event_base
   use eio_data
@@ -93,7 +96,7 @@ contains
     call event%generate (1, [0._default, 0._default])
     call event%set_index (55)
 
-    call eio%output (event, i_prc = 1)
+    call eio%output (event, i_prc = 1)    
     call eio%write (u)
     call eio%final ()
 
@@ -109,14 +112,25 @@ contains
        if (iostat /= 0)  exit
        if (trim (buffer) == "")  cycle
        if (buffer(1:14) == "HepMC::Version")  cycle
-       if (buffer(1:10) == "P 10001 25") &
-            call buffer_blanker (buffer, 32, 55, 78)
-       if (buffer(1:10) == "P 10002 25") &
-            call buffer_blanker (buffer, 33, 56, 79)
-       if (buffer(1:10) == "P 10003 25") &
-            call buffer_blanker (buffer, 29, 53, 78, 101)
-       if (buffer(1:10) == "P 10004 25") &
-            call buffer_blanker (buffer, 28, 51, 76, 99)
+       if (HEPMC2_AVAILABLE) then
+          if (buffer(1:10) == "P 10001 25") &
+               call buffer_blanker (buffer, 32, 55, 78)
+          if (buffer(1:10) == "P 10002 25") &
+               call buffer_blanker (buffer, 33, 56, 79)
+          if (buffer(1:10) == "P 10003 25") &
+               call buffer_blanker (buffer, 29, 53, 78, 101)
+          if (buffer(1:10) == "P 10004 25") &
+               call buffer_blanker (buffer, 28, 51, 76, 99)
+       else if (HEPMC3_AVAILABLE) then
+          if (buffer(1:8) == "P 1 0 25") &
+               call buffer_blanker (buffer, 26, 49, 72)
+          if (buffer(1:8) == "P 2 0 25") &
+               call buffer_blanker (buffer, 26, 49, 73)
+          if (buffer(1:9) == "P 3 -1 25") &
+               call buffer_blanker (buffer, 28, 52, 75)
+          if (buffer(1:9) == "P 4 -1 25") &
+               call buffer_blanker (buffer, 27, 50, 73)
+       end if
        write (u, "(A)") trim (buffer)
     end do
     close (u_file)
@@ -183,31 +197,59 @@ contains
     open (u_file, file = char (sample // ".hepmc"), &
          status = "replace", action = "readwrite")
 
-    write (u_file, "(A)")  "HepMC::Version 2.06.09"
-    write (u_file, "(A)")  "HepMC::IO_GenEvent-START_EVENT_LISTING"
-    write (u_file, "(A)")  "E 66 -1 -1.0000000000000000e+00 &
-         &-1.0000000000000000e+00 &
-         &-1.0000000000000000e+00 42 0 1 10001 10002 0 0"
-    write (u_file, "(A)")  "U GEV MM"
-    write (u_file, "(A)")  "V -1 0 0 0 0 0 2 2 0"
-    write (u_file, "(A)")  "P 10001 25 0 0 4.8412291827592713e+02 &
-         &5.0000000000000000e+02 &
-         &1.2499999999999989e+02 3 0 0 -1 0"
-    write (u_file, "(A)")  "P 10002 25 0 0 -4.8412291827592713e+02 &
-         &5.0000000000000000e+02 &
-         &1.2499999999999989e+02 3 0 0 -1 0"
-    write (u_file, "(A)")  "P 10003 25 -1.4960220911365536e+02 &
-         &-4.6042825611414656e+02 &
-         &0 5.0000000000000000e+02 1.2500000000000000e+02 1 0 0 0 0"
-    write (u_file, "(A)")  "P 10004 25 1.4960220911365536e+02 &
-         &4.6042825611414656e+02 &
-         &0 5.0000000000000000e+02 1.2500000000000000e+02 1 0 0 0 0"
-    write (u_file, "(A)")  "HepMC::IO_GenEvent-END_EVENT_LISTING"
+    if (HEPMC2_AVAILABLE) then
+       write (u_file, "(A)")  "HepMC::Version 2.06.09"
+       write (u_file, "(A)")  "HepMC::IO_GenEvent-START_EVENT_LISTING"
+       write (u_file, "(A)")  "E 66 -1 -1.0000000000000000e+00 &
+            &-1.0000000000000000e+00 &
+            &-1.0000000000000000e+00 42 0 1 10001 10002 0 0"
+       write (u_file, "(A)")  "U GEV MM"
+       write (u_file, "(A)")  "V -1 0 0 0 0 0 2 2 0"
+       write (u_file, "(A)")  "P 10001 25 0 0 4.8412291827592713e+02 &
+            &5.0000000000000000e+02 &
+            &1.2499999999999989e+02 3 0 0 -1 0"
+       write (u_file, "(A)")  "P 10002 25 0 0 -4.8412291827592713e+02 &
+            &5.0000000000000000e+02 &
+            &1.2499999999999989e+02 3 0 0 -1 0"
+       write (u_file, "(A)")  "P 10003 25 -1.4960220911365536e+02 &
+            &-4.6042825611414656e+02 &
+            &0 5.0000000000000000e+02 1.2500000000000000e+02 1 0 0 0 0"
+       write (u_file, "(A)")  "P 10004 25 1.4960220911365536e+02 &
+            &4.6042825611414656e+02 &
+            &0 5.0000000000000000e+02 1.2500000000000000e+02 1 0 0 0 0"
+       write (u_file, "(A)")  "HepMC::IO_GenEvent-END_EVENT_LISTING"
+    else if (HEPMC3_AVAILABLE) then
+       write (u_file, "(A)")  "HepMC::Version 3.01.01"
+       write (u_file, "(A)")  "HepMC::Asciiv3-START_EVENT_LISTING"
+       write (u_file, "(A)")  "E 55 1 4"
+       write (u_file, "(A)")  "U GEV MM"
+       write (u_file, "(A)")  "A 0 alphaQCD -1"
+       write (u_file, "(A)")  "A 0 event_scale 1000"
+       write (u_file, "(A)")  "A 0 signal_process_id 42"
+       write (u_file, "(A)")  "P 1 0 25 0.0000000000000000e+00 &
+            &0.0000000000000000e+00 4.8412291827592713e+02 &
+            &5.0000000000000000e+02 1.2499999999999989e+02 3"
+       write (u_file, "(A)")  "P 2 0 25 0.0000000000000000e+00 &
+            &0.0000000000000000e+00 -4.8412291827592713e+02 &
+            &5.0000000000000000e+02 1.2499999999999989e+02 3"
+       write (u_file, "(A)")  "V -1 0 [1,2]"  
+       write (u_file, "(A)")  "P 3 -1 25 -1.4960220911365536e+02 &
+            &-4.6042825611414656e+02 0.0000000000000000e+00 &
+            &5.0000000000000000e+02 1.2500000000000000e+02 1"
+       write (u_file, "(A)")  "P 4 -1 25 1.4960220911365536e+02 &
+            &4.6042825611414656e+02 0.0000000000000000e+00 &
+            &5.0000000000000000e+02 1.2500000000000000e+02 1"
+       write (u_file, "(A)")  "HepMC::Asciiv3-END_EVENT_LISTING"
+    else
+       call msg_fatal &
+            ("Trying to execute eio_hepmc unit tests without a linked HepMC")
+    end if
     close (u_file)
 
     write (u, "(A)")  "* Initialize test process"
     write (u, "(A)")
 
+    allocate (fallback_model)
     call eio_prepare_fallback_model (fallback_model)
     call eio_prepare_test (event, unweighted=.false.)
 
@@ -245,7 +287,7 @@ contains
        write (u, "(A,I0,A,I0)")  "Found process #", i_prc, &
             " with ID = ", eio%proc_num_id(i_prc)
     end select
-
+    
     call eio%input_event (event, iostat)
 
     call event%write (u)
@@ -264,6 +306,7 @@ contains
 
     call eio_cleanup_test (event)
     call eio_cleanup_fallback_model (fallback_model)
+    deallocate (fallback_model)
 
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: eio_hepmc_2"
