@@ -1,6 +1,6 @@
-! WHIZARD 2.6.4 Aug 23 2018
+! WHIZARD 2.8.2 Oct 24 2019
 !
-! Copyright (C) 1999-2018 by
+! Copyright (C) 1999-2019 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
@@ -28,6 +28,7 @@
 
 module eio_lcio
 
+  use kinds, only: default
   use iso_varying_string, string_t => varying_string
   use io_units
   use string_utils
@@ -51,6 +52,7 @@ module eio_lcio
      logical :: recover_beams = .false.
      logical :: use_alphas_from_file = .false.
      logical :: use_scale_from_file = .false.
+     integer :: n_alt = 0
      type(lcio_writer_t) :: lcio_writer
      type(lcio_reader_t) :: lcio_reader
      type(lcio_run_header_t) :: lcio_run_hdr
@@ -172,6 +174,7 @@ contains
     else
        call msg_fatal ("LCIO: events must be unweighted")
     end if
+    eio%n_alt = data%n_alt
     eio%sample = sample
     if (present (extension)) then
        eio%extension = extension
@@ -230,6 +233,8 @@ contains
     integer, intent(in) :: i_prc
     logical, intent(in), optional :: reading, passed, pacify
     type(particle_set_t), pointer :: pset_ptr
+    real(default) :: sqme_prc, weight
+    integer :: i
     if (present (passed)) then
        if (.not. passed)  return
     end if
@@ -251,6 +256,12 @@ contains
             event%get_beam_file ())
        call lcio_event_set_process_name (eio%lcio_event, &
             event%get_process_name ())
+       do i = 1, eio%n_alt
+          sqme_prc = event%get_sqme_alt(i)
+          weight = event%get_weight_alt(i)
+          call lcio_event_set_alt_sqme (eio%lcio_event, sqme_prc, i)
+          call lcio_event_set_alt_weight (eio%lcio_event, weight, i)
+       end do
        call lcio_event_write (eio%lcio_writer, eio%lcio_event)
        call lcio_event_final (eio%lcio_event)
     else
